/**	$Revision: 1.24 $
	Last updated: $Date: 2002/12/19 22:48:23 $

	Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "AccountsForm.h"
#include "AccountsDB.h"
#include "CoinsPref.h"
#include "EditAccountForm.h"
#include "CurrencyListForm.h"

#include "FreeCoinsRsc.h"

AccountsForm::AccountsForm() : Form (ACCOUNTS_MAIN_FORM)
{
	_edit_trans_form = EditTransactionForm::instance();
	_transactions_form = TransactionsForm::instance();
}

void	AccountsForm::displayTotals()
{
	Char* 			s;
	RectangleType	rp;
	Int32 amount=0;

	switch (preferences.get()->displayed_balance) {
		case 0:
			amount = accounts_db->totals.total;
			break;
		case 1:
			amount = accounts_db->totals.cleared_total;
			break;
		case 2:
			amount = accounts_db->totals.available_total;
	}

	//if our displayed accounts include the income accounts, inverse the sign of the of the total
	if(preferences.get()->visible_acc_types == 2 || preferences.get()->visible_acc_types == 5)
		amount = -amount;

	s=Pence2String(amount);
	RctSetRectangle(&rp, 111, 129, 50, 11);
	WinEraseRectangle(&rp,0);
	DrawCharsRightAlign(s,&rp);
	MemPtrFree(s);
	
	//add some lines at the bottom
	WinDrawLine(0,128,160,128);
	WinDrawLine(0,141,160,141);	
}

void AccountsForm::resetIEAccounts()
{
	//abort if user does not confirm
	MemHandle str_handle = DmGetResource (strRsc,RESET_IE_STR);
	bool rez = FrmCustomAlert(GENERIC_YES_NO,reinterpret_cast<char*> (MemHandleLock (str_handle)),NULL, NULL); 	
	MemHandleUnlock (str_handle);
	DmReleaseResource (str_handle);	
	if(rez) return;	
	MemHandle mh;
	accountStructure as;
	
	//go through all accounts and clear the ones having the type of Income or Expense
  	for(UInt16 record_no=0;;record_no++) {
		if(!DmQueryNextInCategory(accounts_db->getRef(),&record_no,dmAllCategories))
			break;
		mh = accounts_db->getUnpackedRecord(record_no,&as);
		if (as.type == INCOME || as.type == EXPENSE) {
			as.balance = as.cleared_balance = 0;
			accounts_db->packRecord(&as,mh);
		}
	}				
	//will quite possibly have to update the accounts being displayed and the balances
	FrmUpdateForm(form_id,1);
}
void AccountsForm::updatePopupTrigger(void)
{
	PopupListSetSelection(ACC_TYPE_LIST, ACC_TYPE_TRIGGER,
						  preferences.get()->visible_acc_types);
	PopupListSetSelection(ACC_TOTAL_LIST, ACC_TOTAL_TRIGGER,
						  preferences.get()->displayed_balance);
}

Boolean AccountsForm::handleKeyDownEvent(WChar key_id, UInt16 modifiers) 
{		
	ControlPtr ptr;
	
	switch(key_id) {
		case pageUpChr:			// Scroll up key presed?
			//link the hard button functionality to the soft up/down controls
			ptr = (ControlPtr) GetObjectPtr(ACC_PGUP);
			CtlHitControl(ptr);
			break;								
		case pageDownChr:		// Scroll down key presed?						
			ptr = (ControlPtr) GetObjectPtr(ACC_PGDN);
			CtlHitControl(ptr);
			break;
		default:
			return false;
		};
	return true;
}

Boolean AccountsForm::handleOpenEvent()
{	
	//first initialise the table
	ac_tbl.initTable();
	
	//draw form
	FrmUpdateForm (form_id,0);
	return true;
}

Boolean AccountsForm::handleCtlSelectEvent(UInt16 control_id) 
{
	switch(control_id) {
		case ACC_NEW_TRANS:
			if(preferences.get()->def_acc == 0) {
				MemHandle str_handle = DmGetResource (strRsc,DEFAULT_ACC_STR);
				FrmCustomAlert(GENERIC_ERROR,reinterpret_cast<char*> (MemHandleLock (str_handle)), NULL,NULL);
				MemHandleUnlock (str_handle);
				DmReleaseResource (str_handle);
				displayTotals();
			} else {
				accounts_db->selectRecord(preferences.get()->def_acc);
				_edit_trans_form->setRecordIsNew(true);
				_edit_trans_form->setTransIsSched(false);
				popup (_edit_trans_form);
			}
			break;
		case ACC_PGUP:
			if(ac_tbl.pageUp()) FrmUpdateForm(form_id,1);
			break;								
		case ACC_PGDN:		// Scroll down control presed?	
			if(ac_tbl.pageDown()) FrmUpdateForm(form_id,1);
			break;
				
		default:
			return false;
		}
	return true;
}

Boolean AccountsForm::handleTblSelectEvent(tblSelect& tbl_select) 
{
	//save the record on the current row into DetailRecord variable
	ac_tbl.unhighlight();
	if(tbl_select.column==AccountsTable::name_col) {
		// tap in the name column: check if it's in the indented area
		accountStructure a;
		MemHandle handle = accounts_db->getUnpackedRecord(TblGetRowID((TablePtr) GetObjectPtr(ACC_TABLE),tbl_select.row), &a);
		if (last_screenX <= ACC_TBL_INDENT*(a.depth+2)) {
			// yes: check for children
			if (a.children_no > 0) {
				// yes: toggle collapsed state
				a.collapsed = !a.collapsed;
				accounts_db->packRecord(&a, handle);
				FrmUpdateForm(form_id,1);
				return true;
			}
		}
	}
		
	accounts_db->selectRecord( TblGetRowID((TablePtr) GetObjectPtr(ACC_TABLE),tbl_select.row));
    FrmEraseForm (frm);
	popup (_transactions_form);
	return true;
}

Boolean AccountsForm::handleMenuEvent(UInt16 menu_item_id) 
{	
	switch (menu_item_id) {
		case ACC_MENU_NEW:
			if (preferences.get()->def_currency == 0) {
				MemHandle str_handle = DmGetResource (strRsc,NO_DEF_CUR_STR);
				FrmCustomAlert(GENERIC_INFO, reinterpret_cast<char*> (MemHandleLock (str_handle)), NULL, NULL);
				MemHandleUnlock (str_handle);
				DmReleaseResource (str_handle);
				FrmUpdateForm (form_id,0);
				break;
			}
			EditAccountForm::instance()->setRecordIsNew(true);
			if(!osVerGe(palmos_3_5)) FrmEraseForm(frm);
			popup(EditAccountForm::instance());
			break;
		case ACC_MENU_PREF: 
			popup(&_pref_form);
			break;
		case ACC_MENU_CLEAR_IE:
			resetIEAccounts();
			break;
		case ACC_MENU_PURGE:
			popup (&_purge_form);
			break;
		case ACC_MENU_CUR_EDIT:
			popup (CurrencyListForm::instance());
			break;
		case ACC_MENU_ABOUT: {
			FormPtr f = FrmInitForm(ABOUT_FORM);
			FrmSetActiveForm(f);
			FrmDoDialog(f);
			if (frm) {
				FrmSetActiveForm(frm);
				FrmDeleteForm(f);
			}
			FrmUpdateForm (form_id,0);
			break;
		}
		case ACC_MENU_HELP:
			FrmHelp(ACCOUNTS_HELP);
			FrmUpdateForm (form_id,0);
			break;
		default:
			return false;
		}
	return true;	
}

Boolean AccountsForm::handleUpdateEvent(UInt16 update_code) 
{		
	ac_tbl.loadData();

	if(update_code == 0 || update_code == frmRedrawUpdateCode) {
		if(!osVerGe(palmos_3_5)) FrmEraseForm(frm);
		//draw form
		FrmDrawForm(frm);
		TblMarkTableInvalid((TablePtr)GetObjectPtr(ACC_TABLE));
		TblRedrawTable((TablePtr)GetObjectPtr(ACC_TABLE));
		updatePopupTrigger();
	}
	if(update_code & 1) {
		TblMarkTableInvalid((TablePtr)GetObjectPtr(ACC_TABLE));
		TblRedrawTable((TablePtr)GetObjectPtr(ACC_TABLE));
	}
	if(update_code & 2) {
		updatePopupTrigger();
	}
	displayTotals();
	ac_tbl.showScrollArrows();	
	return true;
}

Boolean	AccountsForm::handleCloseEvent() 
{		
	ac_tbl.hideScrollArrows();
	Form::handleCloseEvent();
	return true;
}

Boolean AccountsForm::handlePenDownEvent (Int16 x_coord, Int16 y_coord) 
{
	// it seems that tblSelect doesn't set the pen coordinates in the
	// event structure, therefore I cache the coordinates from the pen
	// events to see where exactly the name column was tapped.
	last_screenX = x_coord;
	last_screenY = y_coord;
	return false;
}

Boolean AccountsForm::handlePopSelect (popSelect& pop_select) 
{
	switch(pop_select.controlID) {
		case ACC_TYPE_TRIGGER:
			if (pop_select.selection != pop_select.priorSelection) {
				preferences.get()->visible_acc_types = pop_select.selection;
				updatePopupTrigger();
				accounts_db->updateVisibility();
				ac_tbl.top_visible_item = 0;
				FrmUpdateForm(form_id,3);
			}
			break;
		case ACC_TOTAL_TRIGGER:
			preferences.get()->displayed_balance = pop_select.selection;
			FrmUpdateForm(form_id,3);
			break;
		default:
			return false;
	}
	return true;
}
