/**	$Revision: 1.13 $
	Last updated: $Date: 2002/09/24 19:46:42 $
	
	Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "AccountsTable.h"

#include "CoinsPref.h" 
#include "AccountsDB.h"
#include "MiscFunc.h"

#include "FreeCoinsRsc.h"

void AccountsTable::drawCurrency(void* table, Int16 row, Int16 column, RectangleType* bounds)
{
	Char*				s;
	accountStructure 	as;
	Int32 amount = 0;

	accounts_db->getUnpackedRecord(TblGetRowID((TablePtr) table,row),&as);
    
	switch (((as.collapsed || preferences.get()->inc_children_total)?3:0)
			 + preferences.get()->displayed_balance ) {
		case 0: amount = as.balance; break;
		case 1: amount = as.cleared_balance; break;
		case 2: amount = as.balance + as.credit_limit; break;
		case 3: amount = as.children_total+as.balance; break;
		case 4: amount = as.children_cleared_total+as.cleared_balance; break;
		case 5: amount = as.children_available_total + as.balance + as.credit_limit; break;
	}

	if (as.credit_account || as.type == INCOME) amount *= -1;
	s = Pence2String(amount);
	DrawCharsRightAlign(s, bounds);

	//free memory allocated by Pence2String
	MemPtrFree(s);
}

void AccountsTable::drawText(void* table, Int16 row, Int16 column, RectangleType* bounds)
{
	accountStructure 	as;
    UInt8 offset, fudge;

	accounts_db->getUnpackedRecord(TblGetRowID((TablePtr) table,row),&as);

  	offset = ACC_TBL_INDENT * as.depth;
	if (as.children_no!=0) {
		fudge = (TblGetRowHeight((TablePtr) table,row)-ACC_TBL_INDENT)/2;
		MemHandle resH = DmGetResource('Tbmp',
			as.collapsed ? BMP_EXPAND : BMP_COLLAPSE);
		BitmapType *bitmap = (BitmapType *)MemHandleLock(resH);
		WinDrawBitmap(bitmap, bounds->topLeft.x+offset,bounds->topLeft.y+fudge);
		MemHandleUnlock(resH);
		DmReleaseResource(resH);
	}
   	offset += ACC_TBL_INDENT + 1;
	WinDrawTruncChars(as.name,StrLen(as.name),
		bounds->topLeft.x+offset,bounds->topLeft.y,bounds->extent.x-offset);
}

Boolean AccountsTable::recordIsValid(UInt16 id)
{
	return (accounts_db->isDisplayable(id));
}

void AccountsTable::loadData()
{
	UInt16		recordNo,row,i;
	
	row = 0;
	i = 0;
	
	initColumns();
	
	for (recordNo=0;
		DmQueryNextInCategory(accounts_db->getRef(),
		&recordNo,accounts_db->current_cat); recordNo++)
	{
		if (!recordIsValid(recordNo)) continue;
		if (i++ < top_visible_item) continue;
		if (row>=rows) continue;
		TblSetRowID(_table, row, recordNo);
		TblSetRowUsable(_table, row, true);
		row++;
	}
	setRowsUnusable(row);
	total_items = i;
}	


void AccountsTable::initColumns()
{	
	//the numbers are drawn with our custom procedure
	TblSetCustomDrawProcedure(_table, name_col, drawText);
	TblSetCustomDrawProcedure(_table, balance_col, drawCurrency);

	//set style for all cells
	for (UInt16 row = 0; row < rows; row++) {
		TblSetItemStyle(_table, row, name_col, customTableItem);
		TblSetItemStyle(_table, row, balance_col, customTableItem);
	}
	
	//set all columns as usable
	TblSetColumnUsable(_table, name_col, true);
	TblSetColumnUsable(_table, balance_col, true);
}		

AccountsTable::AccountsTable() : Table(ACC_TABLE, ACC_PGUP, ACC_PGDN)
{
}
