/**	$Revision: 1.5 $
	Last updated: $Date: 2003/01/29 23:56:50 $
	
	Copyright (C) 2001-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "CalcForm.h"
#include "FreeCoinsRsc.h"
#include "CoinsPref.h"

CalculatorForm* CalculatorForm::_instance = NULL;
CalculatorForm calculator_form;

CalculatorForm* CalculatorForm::instance()
{
	if(_instance==NULL) {
		_instance = &calculator_form;
	}
	return _instance;
}	

// work around GCC problem
void MyLongToFloat(FlpDouble *fd, Int32 x) FLOAT_EM_TRAP(sysFloatEm_d_itod);

CalculatorForm::CalculatorForm() : Form (CALC_FORM)
{}


void CalculatorForm::updateDisplay()
{
	RectangleType rect;
	Char *s;
	FlpCompDouble fcd;
	Boolean savebool;

	RctSetRectangle(&rect, 6, 16, 144, 19);
	WinDrawRectangleFrame(simpleFrame, &rect);
	if (entry[0]) {
		SetFieldTextFromStr(CALC_AMOUNT, entry);
	} else {
		fcd.d = (disp_accu ? accu.d : operand.d) * 100.0;
		if (fcd.d >= 0.0) fcd.d += 0.5;
		else fcd.d -= 0.5;
		// just in case, don't try to display apostrophe as thousands separator
		// since it comes out as a funny box on the calculator font.
		savebool = preferences.get()->use_thousands_sep;
		if (cThousandSep=='\'') preferences.get()->use_thousands_sep=false;
		s = Pence2String(_d_dtoi(fcd.fd));
		preferences.get()->use_thousands_sep = savebool;
		SetFieldTextFromStr(CALC_AMOUNT, s);
		MemPtrFree(s);
	}
}

Boolean CalculatorForm::handleUpdateEvent(UInt16 update_code)
{
	if (!osVerGe(palmos_3_5)) FrmEraseForm(frm);
	FrmDrawForm(frm);
	updateDisplay();
	return true;
}
 
Boolean CalculatorForm::handleOpenEvent()
{
	//draw form
	MemSet(entry, 32, 0);
	MyLongToFloat(&accu.fd, amount);
	operation = '=';
	accu.d = accu.d/100.0;
	disp_accu = true;
	FrmDrawForm(frm);
	dotLabel[0] = cDecimalSep;
	dotLabel[1] = 0;
	CtlSetLabel((ControlType*)FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, CALC_DOT)), dotLabel);
	updateDisplay();
	return true;
}


Boolean CalculatorForm::handleCtlSelectEvent (UInt16 control_id)
{
	Boolean handled = true;

	switch( control_id ) {
	case CALC_OK:
		if (*entry || !disp_accu) {
			doMath('=');
		}
		accu.d *= 100.0;
		if (accu.d >= 0) accu.d += 0.5;
		else accu.d -= 0.5;
		amount = _d_dtoi(accu.fd);
		FrmUpdateForm(caller_form_id, update_code);
		FrmReturnToForm(0);
		break;
	case CALC_CANCEL:
		FrmReturnToForm(0);
		break;
	case CALC_ERASE:
		if (StrLen(entry)>1) entry[StrLen(entry)-1] = 0;
		else StrCopy(entry, "0");
		updateDisplay();
		break;
	case CALC_CLEAR:
		MemSet(entry, 32, 0);
		accu.d = 0.0;
		disp_accu = true;
		operation = '=';
		updateDisplay();
		break;
	case CALC_DIGIT_0: handleButton('0'); break;
	case CALC_DIGIT_1: handleButton('1'); break;
	case CALC_DIGIT_2: handleButton('2'); break;
	case CALC_DIGIT_3: handleButton('3'); break;
	case CALC_DIGIT_4: handleButton('4'); break;
	case CALC_DIGIT_5: handleButton('5'); break;
	case CALC_DIGIT_6: handleButton('6'); break;
	case CALC_DIGIT_7: handleButton('7'); break;
	case CALC_DIGIT_8: handleButton('8'); break;
	case CALC_DIGIT_9: handleButton('9'); break;
	case CALC_DOT:     handleButton(cDecimalSep); break;
	case CALC_EQUALS:  doMath('='); break;
	case CALC_PLUS:    doMath('+'); break;
	case CALC_MINUS:   doMath('-'); break;
	case CALC_TIMES:   doMath('*'); break;
	case CALC_DIV:     doMath('/'); break;
	case CALC_INVERT:  flipSign(); break;
		
	default:
		handled = false;
	}
	return handled;
}

void CalculatorForm::handleButton(Char c)
{
	if (StrLen(entry)>=13)
		return;
	if (c==cDecimalSep && StrChr(entry, cDecimalSep))
		return;
	if (*entry=='0')
		*entry=0;
	entry[StrLen(entry)+1] = 0;
	entry[StrLen(entry)] = c;
	updateDisplay();
}

void CalculatorForm::doMath(Char newop)
{
	FlpCompDouble oldaccu;

	oldaccu.d = accu.d;

	if (disp_accu) {
		if (*entry && disp_accu) {
			MyLongToFloat(&operand.fd, String2Pence(entry));
			operand.d /= 100.0;
		}
		else {
			if (operation=='=' || (newop!=operation && newop!='='))
				operand.d = accu.d;
		}
	}

	if (*entry || newop=='=' || !disp_accu) {
		switch (operation) {
			case '=': accu.d = operand.d; break;
			case '+': accu.d += operand.d; break;
			case '-': accu.d -= operand.d; break;
			case '*': accu.d *= operand.d; break;
			case '/': accu.d /= operand.d; break;
		}
		if (newop!='=') operation = newop;
	}
	else operation = newop;

	if (accu.d < -10000000 || accu.d > 10000000) {
		MemHandle str_handle = DmGetResource (strRsc,OVERFLOW_ERR_STR);
		FrmCustomAlert(GENERIC_ERROR, reinterpret_cast<char*> (MemHandleLock (str_handle)), NULL, NULL);
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
		accu.d = oldaccu.d;
		operation = '=';
	}

	disp_accu = true;
	MemSet(entry, 32, 0);
	updateDisplay();
}

void CalculatorForm::flipSign()
{
	if (!*entry) {
		if (disp_accu)
			accu.d *= -1.0;
		else
			operand.d *= -1.0;
		operation = '=';
	}
	else {
		MyLongToFloat(&operand.fd, String2Pence(entry));
		operand.d /= -100.0;
		disp_accu = false;
		MemSet(entry, 32, 0);
	}
	updateDisplay();
}
