/** $Revision: 1.12 $
    Last updated: $Date: 2002/08/15 07:50:59 $

    Copyright (C) 2001-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "ChooseAccountForm.h"
#include "EditAccountForm.h"
#include "AccountsDB.h"

ChooseAccountForm* ChooseAccountForm::_instance = NULL;
ChooseAccountForm choose_acc_form;

ChooseAccountForm::ChooseAccountForm() : Form (CHOOSE_ACCOUNT_FORM)
{
	allow_no_selection = false; 
	chosen = false;
	//	ErrFatalDisplay ("choose acc form cons");
}

ChooseAccountForm* ChooseAccountForm::instance()
{
	if(_instance == NULL) {
		_instance = &choose_acc_form;
	}
	return _instance;
}	

void ChooseAccountForm::drawAccountName(Int16 itemNum,
	RectangleType *bounds, Char **txt)
{
	Char acc_name[80];
	UInt32 acc_uid;

	acc_uid = accounts_db->getRecordUid(itemNum);
	accounts_db->getFullNameTruncLeftToFit(acc_uid, bounds->extent.x, acc_name);
	WinDrawChars(acc_name, StrLen(acc_name),
		bounds->topLeft.x, bounds->topLeft.y);
}

Boolean ChooseAccountForm::handleOpenEvent()
{
	ListPtr list= (ListPtr) GetObjectPtr(CHOOSE_ACC_LIST);
	Int16 sel;

	LstSetDrawFunction(list, drawAccountName);
	LstSetListChoices(list, NULL, DmNumRecordsInCategory(accounts_db->getRef(),
														 dmAllCategories));
	sel = noListSelection;
	if (chosen_account) {
		if(DmFindRecordByID(accounts_db->getRef(),chosen_account,(UInt16*)&sel))
			sel = noListSelection;
	}
	LstSetSelection(list, sel);
	if (sel!=noListSelection)
		LstMakeItemVisible(list, sel);
	if (allow_no_selection)
		FrmShowObject(frm, FrmGetObjectIndex(frm, CHOOSE_ACC_NONE));
	if (allow_new_account)
		FrmShowObject (frm, FrmGetObjectIndex (frm, CHOOSE_ACC_NEW_ACC));
	FrmDrawForm (frm);
	return true;
}

Boolean ChooseAccountForm::handleUpdateEvent (UInt16 update_code)
{
	FrmDrawForm (frm);
	return true;
}

/*if a new account is created, a goto path through forms is taken - see EditAccountForm for more details */
Boolean ChooseAccountForm::handleCtlSelectEvent(UInt16 control_id)
{
	bool handled=true;
	ListPtr list= (ListPtr) GetObjectPtr(CHOOSE_ACC_LIST);
	switch(control_id) {
		case CHOOSE_ACC_OK: {
			handled = true;
			Int16 sel = LstGetSelection(list);
			if((sel == noListSelection) && !allow_no_selection) break;
			if((sel == noListSelection) && allow_no_selection) {
				//user made empty selection
				chosen_account = 0;
			} else {
				UInt32 acc_uid = accounts_db->getRecordUid(sel);
				// first check that the chosen account is not the same
				// as the current one
				if(accounts_db->isRecordSelected() && (accounts_db->getSelectedUid() == acc_uid)) {
					MemHandle str_handle = DmGetResource (strRsc,NOT_CUR_ACC_STR);
					FrmCustomAlert(GENERIC_ERROR,reinterpret_cast<char*> (MemHandleLock (str_handle)),NULL, NULL);
					MemHandleUnlock (str_handle);
					DmReleaseResource (str_handle);
					break;
				}
				chosen_account = acc_uid;
			}
			chosen = true;		
			allow_new_account = false;
			FrmUpdateForm(caller_form_id,0);		
			FrmReturnToForm(0);
			break;
		}
							
		case CHOOSE_ACC_CANCEL:
			//cancel button was pressed;
			//simply return to the main form
			allow_new_account = false;
			FrmReturnToForm(0);
			break;

		case CHOOSE_ACC_NONE:
			LstSetSelection(list, noListSelection);
			break;
		case CHOOSE_ACC_NEW_ACC:
			EditAccountForm::instance()->setRecordIsNew(true);
			if(!osVerGe(palmos_3_5)) FrmEraseForm(frm);
			EditAccountForm::instance()->setCallerFormId (0);
			FrmGotoForm(EDIT_ACC_FORM);
			break;
			
		default:
			handled = false;
	};
	return handled;
}

Boolean ChooseAccountForm::handleKeyDownEvent(WChar key_id, UInt16 modifiers)
{
	bool handled = true;
	ListPtr list= (ListPtr) GetObjectPtr(CHOOSE_ACC_LIST);

	switch (key_id) {
	case pageUpChr:
		LstScrollList(list, winUp, 9);
		break;
		
	case pageDownChr:
		LstScrollList(list, winDown, 9);
		break;

	default:
		handled = false;
		break;
	}
	return handled;
}
