/**	$Revision: 1.28 $
	Last updated: $Date: 2002/09/20 12:21:30 $
	
	Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef __COINS_PREFERENCES__
#define __COINS_PREFERENCES__

#include <PalmOS.h>

#include "MiscFunc.h"

/// possible payment methods for transactions
typedef enum { pm_card=0, pm_cash=1, pm_cheque=2, pm_so=3, pm_dd=4,
	 pm_transfer=5, pm_online=6, pm_direct_deposit=7, pm_atm=8, pm_other=9 } Payment_method;

///structure containing the program preferences
typedef struct {
	///when the scheduled transactions were last updated
	DateType last_update;	
	///account types the user wants to see
	UInt16 visible_acc_types;
	///default view in the transaction list
	UInt16 trans_view;
	///default account for quick transactions (0 if not set)
	UInt32 def_acc;
	///default currency used by the program (UID from currency database)
	UInt32 def_currency;
	///default payment method in transactions
	Payment_method payment_method;
	/** field indicating what is the balance displayed for accounts 
	    it can be 0 for total, 1 for cleared, 2 for available */
	unsigned displayed_balance   :2;
	///true if the payee is displayed in transaction list, false otherwise
	unsigned disp_payee			:1;
	///true if we want to display a cleared column in the transactions table, false otherwise
	unsigned disp_cleared_col	:1;
	///if true the last view will allways be remembered
	unsigned remember_trans_view	:1;
	///default cleared state for transactions
	unsigned trans_cleared		:1;
	///true if user wants thousands separators to display
	unsigned use_thousands_sep	:1;
    ///true if user wants pasword check on program start
    unsigned check_pass :1;
    ///true if user wants the children total to be included in the account balance, even if the account is expanded
    unsigned inc_children_total :1;
	///reserved for future extensions
	unsigned reserved 			:7;
} CoinsPrefType;


///manipulates the program preferences
class CoinsPref 
{
 private:
	///the structure holds the app's preferences
	CoinsPrefType 	coins_pref;	
	
	///id for the pref structure above
	UInt16	id;	
	/// version
	Int16 	ver;
	
 public:
	/// the constructor sets the values for the id and version
	CoinsPref();
		 
	///@return pointer to the current preferences:
	CoinsPrefType* get();
	
	///retrieve the preferences into coins_pref
	void read();

	///save the preferences from coins_pref
	void save();


	///assigns default values to coins_pref
	void setDefault();
};

extern CoinsPref preferences;

#endif
