/**	$Revision: 1.7 $
	Last updated: $Date: 2002/09/27 16:06:26 $
	
	Copyright (C) 2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <PalmOS.h>

#include "CurrencyDB.h"
#include "AccountsDB.h"
#include "CoinsPref.h"
#include "MiscFunc.h" //macro defs
#include "FreeCoinsRsc.h"

CurrencyDB* currency_db;

void CurrencyDB::unpackRecord (currencyStructure* currency, packedCurrencyStructure* packed_currency)
{
	currency->exchange_rate = packed_currency->exchange_rate;
	Char* s = packed_currency->text;
	currency->code = s;
	s += StrLen (s)+1;
	currency->description = s;
}

void CurrencyDB::packRecord (currencyStructure* currency, MemHandle db_entry)
{
	UInt32	length = 0;
	MemPtr	s;
	UInt32	offset = 0;

	length = sizeof(currency->exchange_rate) + StrLen (currency->code) + StrLen (currency->description) + 2;
	

	// resize the record if necessary and write the data
	if (MemHandleResize(db_entry, length) == 0) {
		s = MemHandleLock(db_entry);
		writeAndAdvance(s, offset, &currency->exchange_rate);
		writeAndAdvance(s, offset, currency->code);
		writeAndAdvance(s, offset, currency->description);
		MemHandleUnlock(db_entry);
	} else {
		ErrFatalDisplay ("resizing failed");
	}	
}

CurrencyDB::CurrencyDB() : Database <currencyStructure, packedCurrencyStructure> (
	CURRENCYDB_TYPE, FREE_COINS_CREATOR, CURRENCYDB_NAME)
{
	current_cat=dmAllCategories;
}

UInt16	CurrencyDB::newRecord(currencyStructure* c)
{
	MemHandle h,p;
	UInt16 index=0;
	MemHandle str_handle;

	//first of all do the sanity checks - need to check for uniqueness of code
	if (checkForExistingCode (c->code)) {
		str_handle = DmGetResource (strRsc,DUPLICATE_CODE_STR);	
		ErrFatalDisplay(reinterpret_cast<char*> (MemHandleLock (str_handle)));
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
	}
		
	//create the record
	h=DmNewHandle(ref,1);
	if(!h) {
		str_handle = DmGetResource (strRsc,NEW_CURRENCY_FAILED_STR);	
		ErrFatalDisplay(reinterpret_cast<char*> (MemHandleLock (str_handle)));
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
	}

	packRecord(c,h);	
	p= (MemHandle) MemHandleLock(h);
	index=DmFindSortPosition(ref,p, 0, (DmComparF *) compareRecords, 0);
	MemPtrUnlock(p);
	DmAttachRecord(ref,&index,h,0);
	selectRecord(index);
	
	//check whether this is the only record
	UInt16 count = DmNumRecordsInCategory (ref,dmAllCategories);
	if (count == 1)
		preferences.get()->def_currency = getRecordUid (index);
	
	return index;
}

bool CurrencyDB::deleteRecord(UInt16 id)
{
	if (checkCurrencyDependency (id)) {
		return false;	
	}
	return Database<currencyStructure, packedCurrencyStructure>::deleteRecord(id);	
}

bool CurrencyDB::checkCurrencyDependency (UInt16 id)
{
	MemHandle str_handle;
	UInt32 uid = getRecordUid (id);

	if (preferences.get()->def_currency == uid) {
		str_handle = DmGetResource (strRsc,CURRENCY_IS_DEFAULT_STR);
		FrmCustomAlert(GENERIC_INFO,reinterpret_cast<char*> (MemHandleLock (str_handle)),NULL,NULL);
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
		return true;
	}
	
	accountStructure as;
	for (UInt16 record_id = 0; record_id<DmNumRecordsInCategory (accounts_db->getRef(),dmAllCategories); record_id++){
		accounts_db->getUnpackedRecord (record_id, &as);
		if(as.currency==uid) {
			str_handle = DmGetResource (strRsc,ACC_USING_CURRENCY_STR);
			FrmCustomAlert(GENERIC_INFO,reinterpret_cast<char*> (MemHandleLock (str_handle)),NULL,NULL);
			MemHandleUnlock (str_handle);
			DmReleaseResource (str_handle);
			return true;
		}	
	}
	return false;
}

bool CurrencyDB::checkForExistingCode (Char* code, UInt32 ignore_uid = 0)
{
	currencyStructure cs;
	for (UInt16 record_id = 0; record_id<DmNumRecordsInCategory (ref,dmAllCategories); record_id++){
		if (ignore_uid != 0 && getRecordUid (record_id) == ignore_uid) continue;
		getUnpackedRecord (record_id, &cs);
		if(StrCompare (code, cs.code) == 0) return true;
	}
	return false;
}


Int16 CurrencyDB::compareRecords(packedCurrencyStructure* rec1,packedCurrencyStructure* rec2,Int16 other,SortRecordInfoPtr sr1,SortRecordInfoPtr sr2,MemHandle app_info)
{
	return StrCompare (rec1->text, rec2->text);
}

double CurrencyDB::getExchangeRate (UInt32 currency1, UInt32 currency2)
{
	currencyStructure c1, c2;
	getUnpackedRecord (currency1, &c1);
	getUnpackedRecord (currency2, &c2);
	
	if (currency1 == currency2) return 1.0;
 	if (currency1 == preferences.get()->def_currency)
		return c2.exchange_rate;
	if (currency2 == preferences.get()->def_currency)
		return 1.0/c1.exchange_rate;

	return c2.exchange_rate/c1.exchange_rate;
}

void CurrencyDB::adjustExchangeRate (UInt32 src_currency, UInt32 dest_currency, double rate)
{
	currencyStructure c1, c2;

	MemHandle handle1 = getUnpackedRecord (src_currency, &c1);

	if (src_currency == dest_currency) {
		c1.exchange_rate = 1.0;
		packRecord (&c1, handle1);
		return;
	}

	MemHandle handle2 = getUnpackedRecord (dest_currency, &c2);
	
 	if (src_currency == preferences.get()->def_currency) {
		c2.exchange_rate = rate;
		packRecord (&c2, handle2);
		return;
	}
	if (dest_currency == preferences.get()->def_currency) {
		c1.exchange_rate = 1.0/rate;
		packRecord (&c1,handle1);	
		return;
	}

	//none of the currencies are part of the default currency. We cannot alter the rate
	MemHandle str_handle = DmGetResource (strRsc,NO_DEF_CURRENCY_STR);
	FrmCustomAlert(GENERIC_INFO, reinterpret_cast<char*> (MemHandleLock (str_handle)), NULL, NULL);
	MemHandleUnlock (str_handle);
	DmReleaseResource (str_handle);	
}			

void CurrencyDB::newDefaultCurrency (UInt32 currency)
{
	currencyStructure cs;
	MemHandle rec_h;

	if(preferences.get()->def_currency == 0) {
		//update the new default currency
		rec_h = getUnpackedRecord (currency, &cs);
		cs.exchange_rate = 1.0;
		packRecord (&cs, rec_h);
		return;
	}
		
	UInt32 rec_uid;
	for (UInt16 record_id = 0; record_id<DmNumRecordsInCategory (ref,dmAllCategories); record_id++){
		rec_uid = getRecordUid (record_id);
		/* need to skip the future default currency as its exchange rate to the previous default currency is used in calculating
		   the exchange rates for remaining records */
		if (rec_uid == currency) continue; 
		rec_h = getUnpackedRecord (record_id, &cs);
		cs.exchange_rate = getExchangeRate (currency, rec_uid);
		packRecord (&cs, rec_h);
	}
	//update the new default currency
	rec_h = getUnpackedRecord (currency, &cs);
	cs.exchange_rate = 1.0;
	packRecord (&cs, rec_h);
}

void CurrencyDB::getDescriptionTruncLeftToFit(UInt32 uid, Int16 width, Char* buffer)
{
	currencyStructure c;

	getUnpackedRecord (uid, &c);
	StrCopy (buffer, c.code);
	StrCat (buffer,": ");
	StrCat (buffer, c.description);
	
	Char ellipsis;
	int i;

	for (i=StrLen (buffer)-1; i>=0; i--) {
		width -= FntCharWidth(buffer[i]);
		if (width<0) break;
	}
	if (width<0) {
		ChrHorizEllipsis(&ellipsis);
		width -= FntCharWidth(ellipsis);
		while (width<0) {
			width += FntCharWidth(buffer[i]);
			i++;
		}
		buffer[i] = ellipsis;
		buffer[i+1] = 0;
	}
}
