/** $Revision: 1.36 $
    Last updated: $Date: 2003/01/30 00:08:32 $

    Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "EditAccountForm.h"
#include "FormManager.h"
#include "AccountsDB.h"
#include "CurrencyDB.h"
#include "CoinsPref.h"
#include "MiscFunc.h"
#include "FreeCoinsRsc.h"
#include "CalcForm.h"

EditAccountForm* EditAccountForm::_instance = NULL;
EditAccountForm edit_acc_form;

EditAccountForm::EditAccountForm() : Form (EDIT_ACC_FORM)
{
	choose_account_form = ChooseAccountForm::instance();
	_note_form = NoteForm::instance();
}

EditAccountForm* EditAccountForm::instance()
{
	if(_instance == NULL) {
		_instance = &edit_acc_form;
	}
	return _instance;
}

Boolean EditAccountForm::save()
{	
	Int32 newbal;
	UInt16 index;
	Int8 depth_increment = 0;
	MemHandle h;   

    //if the balance is invalid it will be set to 0 by default
   	newbal=String2Pence((Char*) GetTextPtr(EDIT_ACC_BALANCE));

	//get the credit limit
   	account.credit_limit=String2Pence((Char*) GetTextPtr(EDIT_ACC_CREDIT));

	//get the description
   	account.description = GetTextPtr(EDIT_ACC_DESCRIPTION);
	if(!account.description) account.description = "";
	//same for account name; save is forbidden if no name is set
   	account.name = GetTextPtr(EDIT_ACC_NAME);
   	if(!account.name) return false;

	if (!account.note) account.note = "";
	
	ListPtr type_list = reinterpret_cast<ListPtr>(FrmGetObjectPtr(frm,getObjectIndex(EDIT_ACC_TYPE_LIST)));
	account.type = (AccountType)LstGetSelection(type_list);
	account.credit_account=FrmGetControlValue(frm,FrmGetObjectIndex(frm,EDIT_ACC_CREDIT_CBX));
	
	if (account.credit_account) {
		newbal *= -1;
	}

	if (record_is_new) {
		account.balance=newbal;
		account.cleared_balance=newbal;
		account.children_no = 0;
		ListPtr cur_list = reinterpret_cast<ListPtr>(GetObjectPtr(EDIT_ACC_CUR_LIST));		
		account.currency = currency_db->getRecordUid (LstGetSelection (cur_list));
		index=accounts_db->newRecord(&account);
	} else { //write the data
		//check if the account has been reparented. If so, update the old and
		//new parent's children_no and children total, and the account's depth
		if (account.parent_uid!=prev_parent_uid || account.depth!=prev_depth) {
			accountStructure    parent_acct;

			// decrease old parent's children count
			if (prev_depth!=0) {
				h = accounts_db->getUnpackedRecord(prev_parent_uid,
					&parent_acct);
				parent_acct.children_no--;
				accounts_db->packRecord(&parent_acct, h);

				// subtract the balance from the old parent's children totals
				accounts_db->incrementBalance(prev_parent_uid,
					-account.balance-account.children_total,
					-account.cleared_balance-account.children_cleared_total,
					true);
			}

			// increase new parent's children count and set new depth
			if (account.depth!=0) {
				h = accounts_db->getUnpackedRecord(account.parent_uid,
					&parent_acct);
				parent_acct.children_no++;
				accounts_db->packRecord(&parent_acct, h);

				// add the balance to the new parent's children totals
				accounts_db->incrementBalance(account.parent_uid,
					account.balance+account.children_total,
					account.cleared_balance+account.children_cleared_total,
					true);
			}
		}
		depth_increment = account.depth - prev_depth;
		account.depth = prev_depth;

		accounts_db->packRecord(&account,rech);

		// apply depth change to the account and all descendants
		if (depth_increment) {
			accountStructure a;
			UInt16 rec_id = accounts_db->getSelectedRecord();

			while (DmQueryNextInCategory(accounts_db->getRef(),
						&rec_id, dmAllCategories)) {
				h = accounts_db->getUnpackedRecord(rec_id, &a);
				if (a.depth<=prev_depth &&
					rec_id!=accounts_db->getSelectedRecord()) {
					break;
				}
				a.depth += depth_increment;
				accounts_db->packRecord(&a,h);
				rec_id++;
			}
		}

		//better late than never, this sets the busy bit
		DmGetRecord(accounts_db->getRef(), accounts_db->getSelectedRecord());

		//release record clears the busy bit and set the dirty bit
		DmReleaseRecord(accounts_db->getRef(), accounts_db->getSelectedRecord(), true); 

		if (newbal!=account.balance) { 
			accounts_db->updateBalance(accounts_db->getSelectedUid(),newbal);
		}


		//sort the records in the db - just in case the name changed
		UInt32 uid = accounts_db->getSelectedUid();
		DmInsertionSort(accounts_db->getRef(),
			(DmComparF *) accounts_db->compareRecords,0);
		// Reselect the record in case sorting changed its position.
		// The database should probably track the selected record by its uid,
		// but until that happens, this will do.
		accounts_db->selectRecord(uid);
	}
	accounts_db->updateVisibility();
	return true;
}

void EditAccountForm::drawCurrencyItem(Int16 item_no, RectangleType* bounds, Char** txt)
{
	currencyStructure cs;
	currency_db->getUnpackedRecord ((UInt16)item_no, &cs);
	WinDrawChars(cs.code, StrLen(cs.code),bounds->topLeft.x, bounds->topLeft.y);
}

Boolean EditAccountForm::handlePopSelect (popSelect& pop_select)
{
	switch (pop_select.controlID) {
		case EDIT_ACC_CUR_TRIGGER:
			if(pop_select.selection != pop_select.priorSelection){
				updateCurPopupTrigger (pop_select.selection);
				break;
			}
			break;
			
		default:
			return false;
	}
	return true;
}
	
void EditAccountForm::updateCurPopupTrigger (UInt16 selection)
{
	ControlPtr trig = reinterpret_cast<ControlPtr>(FrmGetObjectPtr(frm, getObjectIndex(EDIT_ACC_CUR_TRIGGER)));
	currencyStructure cs;
	currency_db->getUnpackedRecord (selection, &cs);
    CtlSetLabel(trig, cs.code);	
}

void EditAccountForm::setCurrencyListSelection (UInt16 selection)
{
    ListPtr list = reinterpret_cast<ListPtr>(FrmGetObjectPtr(frm, getObjectIndex(EDIT_ACC_CUR_LIST)));
    LstSetSelection(list, selection);
	LstMakeItemVisible (list, selection);
	updateCurPopupTrigger (selection);
}

void EditAccountForm::init()
{
	Char*	s;
	selected_cat=accounts_db->findCategory("Visible");

	if (record_is_new) {
		MemHandle str_handle = DmGetResource (strRsc,NEW_ACC_STR);
		FrmCopyTitle(frm,reinterpret_cast<char*> (MemHandleLock (str_handle)));
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);

		MemSet(&account, sizeof(accountStructure), 0);
		account.credit_account = false;
		setParentAccount(0); //by default no parent
		account.type = ASSET;
		account.name = "";
		account.note = "";
		ListPtr list = reinterpret_cast<ListPtr>(GetObjectPtr(EDIT_ACC_CUR_LIST));
		LstSetDrawFunction(list, drawCurrencyItem);
		LstSetListChoices(list, NULL, DmNumRecordsInCategory(currency_db->getRef(), dmAllCategories));
		setCurrencyListSelection (currency_db->getRecordId (preferences.get()->def_currency));
		FrmShowObject (frm, getObjectIndex (EDIT_ACC_CUR_TRIGGER));
		FrmHideObject (frm, getObjectIndex (EDIT_ACC_CUR_LABEL));
	} else {
		MemHandle str_handle = DmGetResource (strRsc,ACC_DETAILS_STR);
		FrmCopyTitle(frm, reinterpret_cast<char*> (MemHandleLock (str_handle)));
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
		FrmShowObject(frm, getObjectIndex(EDIT_ACC_DELETE));
		rech = accounts_db->getUnpackedRecord(&account);				

		//set the credit checkbox
		FrmSetControlValue(frm,FrmGetObjectIndex(frm,EDIT_ACC_CREDIT_CBX),account.credit_account);
				
		SetFieldTextFromStr(EDIT_ACC_DESCRIPTION,account.description);
		SetFieldTextFromStr(EDIT_ACC_NAME,account.name);

		s=Pence2String((account.credit_account?-1:1)*account.balance);
		SetFieldTextFromStr(EDIT_ACC_BALANCE,s);
		MemPtrFree(s);
		s=Pence2String(account.credit_limit);
		SetFieldTextFromStr(EDIT_ACC_CREDIT,s);
		//free memory allocated by Pence2String
		MemPtrFree(s);

		//set the currency label
		currencyStructure cs;
		currency_db->getUnpackedRecord (account.currency, &cs);
		FrmCopyLabel (frm, EDIT_ACC_CUR_LABEL, cs.code);
		FrmHideObject (frm, getObjectIndex (EDIT_ACC_CUR_TRIGGER));
        FrmShowObject (frm, getObjectIndex (EDIT_ACC_CUR_LABEL));
		

		if(account.depth != 0)
			setParentAccount(account.parent_uid); //parent is an account
		else
			setParentAccount(0); //no parent

		prev_parent_uid = account.parent_uid;
		prev_depth = account.depth;
	}
    updateTypeSelection();
}

void EditAccountForm::updateTypeSelection(void)
{
	PopupListSetSelection(EDIT_ACC_TYPE_LIST, EDIT_ACC_TYPE_TRIGGER,
		(UInt8)account.type);
}

void	EditAccountForm::setParentAccount(UInt32 id)
{
	if(id) { //if the user selected an account
		accountStructure as;
		account.parent_uid = id;
		accounts_db->getUnpackedRecord(id, &as);
		account.depth = as.depth + 1;
		accounts_db->getFullNameTruncLeftToFit(account.parent_uid, 110,
			parent_name);
	} else {
		MemHandle str_handle = DmGetResource (strRsc,TOP_LEVEL_STR);	
		StrCopy(parent_name, reinterpret_cast<char*> (MemHandleLock (str_handle)));
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
		account.depth = account.parent_uid = 0;
	}
	CtlSetLabel((ControlType*) GetObjectPtrByForm(FrmGetActiveForm(),EDIT_ACC_PARENT),parent_name);
}

Boolean EditAccountForm::handleOpenEvent()
{
	FrmDrawForm(frm);
	init();
	return true;
}

Boolean EditAccountForm::handleUpdateEvent (UInt16 update_code)
{
	accountStructure acc;

	if (update_code == 5) {
		//handle note update case
		if(_note_form->updatedData (form_id))
			account.note = _note_form->getNote();
		return true;
	}		

	if(choose_account_form->chosen) {
		UInt32 this_uid, parent_uid;
		Boolean bad_choice = false;
		choose_account_form->chosen = false;
		if (!record_is_new) {
			this_uid = accounts_db->getRecordUid(accounts_db->getSelectedRecord());	
			parent_uid = choose_account_form->chosen_account;
			while (parent_uid && parent_uid != this_uid) {
				accounts_db->getUnpackedRecord(parent_uid, &acc);
				if (acc.depth==0) break;
				parent_uid = acc.parent_uid;
			}
			if (parent_uid == this_uid) bad_choice = true;
		}
		if (bad_choice) {
			MemHandle str_handle = DmGetResource (strRsc,REC_ACC_STR);
			FrmCustomAlert(GENERIC_ERROR, reinterpret_cast<char*> (MemHandleLock (str_handle)), NULL, NULL);
			MemHandleUnlock (str_handle);
			DmReleaseResource (str_handle);
			choose_account_form->chosen_account = 0;
		} else {
			setParentAccount(choose_account_form->chosen_account);
		}
		// inherit type and credit account flag from parent
		if (choose_account_form->chosen_account) {
			accounts_db->getUnpackedRecord(choose_account_form->chosen_account, &acc);
			account.type = acc.type;
			if (account.credit_account != acc.credit_account) {
				Int32 bal;
				Char *s;
				bal = -String2Pence((Char*) GetTextPtr(EDIT_ACC_BALANCE));
				s = Pence2String(bal);
				SetFieldTextFromStr(EDIT_ACC_BALANCE,s);
				MemPtrFree(s);	
				account.credit_account = acc.credit_account;
			}
			FrmSetControlValue(frm,FrmGetObjectIndex(frm,EDIT_ACC_CREDIT_CBX),account.credit_account);
			updateTypeSelection();
		}
	}

	if (update_code == CalculatorForm::instance()->update_code) {
		Char *s;
		s = Pence2String(CalculatorForm::instance()->amount);
		SetFieldTextFromStr(EDIT_ACC_BALANCE,s);
		MemPtrFree(s);	
	}

	if (!osVerGe(palmos_3_5)) FrmEraseForm(frm);
	FrmDrawForm(frm);		

	return true;	
}

/* if we have been called from chooseaccountform (caller_Form_id is 0), then we move back to that form with goto (thus
   re-creating it) and we set the caller_id for the form to transactions_form. While the popup() method would be cleaner,
   we cannot load a previously loaded form (chooseaccountform would have to be loaded twice: once from edittransaction and once from
   editaccount). */
Boolean EditAccountForm::handleCtlSelectEvent(UInt16 control_id)
{
	switch(control_id) {
		case EDIT_ACC_CREDIT_CBX: {
			Int32 bal;
			Char *s;
			bal = -String2Pence((Char*) GetTextPtr(EDIT_ACC_BALANCE));
			s = Pence2String(bal);
			SetFieldTextFromStr(EDIT_ACC_BALANCE,s);
			MemPtrFree(s);	
			break;
		}
		case EDIT_ACC_OK: {
			//if something went wrong do nothing
			UInt32 prev_acc;
			if(accounts_db->isRecordSelected()) {
				prev_acc = accounts_db->getSelectedUid();
			} else prev_acc = 0;
			
			if(! save()) break;
			if(caller_form_id ==0 ) {
				ChooseAccountForm::instance()->setCallerFormId (EDIT_TRANS_FORM);
				//need to re-select previously selected account
				accounts_db->selectRecord (prev_acc);
				FrmGotoForm (CHOOSE_ACCOUNT_FORM);
			} else {
				FrmUpdateForm(caller_form_id,0);				
				FrmReturnToForm(0);
			}
			break;
		}
		case EDIT_ACC_PARENT:
			choose_account_form->allow_no_selection = true;	
			choose_account_form->chosen_account = account.parent_uid;
			choose_account_form->allow_new_account = false;
			popup(choose_account_form);
			break;
			
		case EDIT_ACC_CANCEL:
			//cancel button was pressed;
			//simply return to the main form
			if(caller_form_id ==0 ) {
				ChooseAccountForm::instance()->setCallerFormId (EDIT_TRANS_FORM);
				FrmGotoForm (CHOOSE_ACCOUNT_FORM);
			} else {
				FrmUpdateForm (caller_form_id,0);
				FrmReturnToForm(0);
			}
			
			break;
		case EDIT_ACC_DELETE:
			//delete button
			if (record_is_new) {
				// we shouldn't be here, the button is invisible.
				// Just to be safe, though...
				return false;
			} else {
				// quick and dirty anti-crash precaution: don't allow
				// accounts with children to be deleted
				accountStructure a;
				accounts_db->getUnpackedRecord(&a);
				if (a.children_no) {
					MemHandle str_handle = DmGetResource (strRsc,DEL_CHILDREN_STR);
					FrmCustomAlert(GENERIC_ERROR,reinterpret_cast<char*> (MemHandleLock (str_handle)) , NULL, NULL);
					MemHandleUnlock (str_handle);
					DmReleaseResource (str_handle);
					break;
				}
			
			MemHandle str_handle = DmGetResource (strRsc,ACCOUNT_STR);
			bool rez=FrmCustomAlert(DELETE_ALERT,reinterpret_cast<char*> (MemHandleLock (str_handle)),NULL,NULL);
			MemHandleUnlock (str_handle);
			DmReleaseResource (str_handle);
			if (rez) break;

			// we are deleting the account currently displayed; we shall need to destroy the transactions form 
			// and return to the accounts list
            //delete the transaction form
			FrmEraseForm(FrmGetFormPtr(TRANSACTIONS_FORM));				
			FrmDeleteForm(FrmGetFormPtr(TRANSACTIONS_FORM));				
            //return to the accounts form
			accounts_db->deleteRecord(accounts_db->getSelectedRecord());
			FrmUpdateForm(ACCOUNTS_MAIN_FORM,0);		
            //            AccountsForm::instance()->handleUpdateEvent (0);
			FrmReturnToForm(ACCOUNTS_MAIN_FORM);
			}
			break;				
		case EDIT_ACC_CALC:
			CalculatorForm::instance()->amount = String2Pence((Char*) GetTextPtr(EDIT_ACC_BALANCE));
			popup(CalculatorForm::instance());
			break;
		case EDIT_ACC_NOTE:
			_note_form->initForm (account.name, account.note);
			popup (_note_form);
			break;
		default:
			return false;
	};

	return true;
}
