/** $Revision: 1.21 $
    Last updated: $Date: 2002/09/20 12:21:31 $

    Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef __EDIT_ACCOUNT_FORM__
#define __EDIT_ACCOUNT_FORM__

#include <PalmOS.h>

#include "Form.h"
#include "AccountsDB.h"
#include "ChooseAccountForm.h"
#include "NoteForm.h"

///the form is used to create a new transaction or to modify an existing one, depending on the record_is_new member
class EditAccountForm : public Form
{	
public:
	///constructor
	EditAccountForm();

	///returns singleton reference to this object
	static EditAccountForm* instance() FORM_SECTION1;
	
	void setRecordIsNew(Boolean b) { record_is_new = b; }
	/// callback for custom list drawing
	static void drawCurrencyItem(Int16 item_no, RectangleType* bounds, Char** txt) FORM_SECTION1;
private:
	/// the selected category
	UInt16	selected_cat;
	///buffer to store the parent name
	Char	parent_name[55];

	///holds the data being manipulated
	accountStructure account;
	///handle to the currently edited account record (used for saving the record)
	MemHandle	rech;	
	///previous parent of the account
	UInt32 prev_parent_uid;
	///previous depth of the account
	UInt8 prev_depth;
	/// this is set to true if we want to create a new account and to false if we want the 'details' functionality
	Boolean record_is_new;
	///contains the id of the last saved transaction
	UInt16 trans_id;
	///local choose acc form
	ChooseAccountForm* choose_account_form;

	///reference to this object
	static EditAccountForm* _instance;

	///form to handle notes
	NoteForm* _note_form;

	///saves the changes to the database
	Boolean save() FORM_SECTION1;
	///set the values of various form fields from the database
	void	init() FORM_SECTION1;
	/**	saves the data provided by ChooseAccountForm and updates the screen
		@param id of the account/category to be displayed	*/
	void	setParentAccount(UInt32 id) FORM_SECTION1;
	/// reflect change in account type selection on popup trigger and list
	void 	updateTypeSelection() FORM_SECTION1;
	/**  set the currency trigger and selects the right item from currency list
		 @param selection id of the currency to be selected */
	void setCurrencyListSelection (UInt16 selection) FORM_SECTION1;
	/**  updates the currency trigger by writing the text corresponding to a given selection
		 @param selection ID of the currency to be displayed */
	void updateCurPopupTrigger (UInt16 selection) FORM_SECTION1;

	///draws form and sets default values
	Boolean handleOpenEvent() FORM_SECTION1;
	/// event handler for controls in  the form
	Boolean handleCtlSelectEvent(UInt16 control_id) FORM_SECTION1;
	///handles form updating
	Boolean handleUpdateEvent(UInt16 update_code) FORM_SECTION1;	
	///sets value of record_is_new member
	/**  handle the currency popup select event 
	     @param pop_select event data captured */
	Boolean handlePopSelect (popSelect& pop_select) FORM_SECTION1;
};


#endif
