/**	$Revision: 1.5 $
	Last updated: $Date: 2003/01/30 00:27:43 $

	Copyright (C) 2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "EditRateForm.h"
#include "CurrencyDB.h"
#include "CoinsPref.h"
#include "FreeCoinsRsc.h"

EditRateForm::EditRateForm() : Form (EDIT_RATE_FORM)
{
	results.dirty = false;
}

void EditRateForm::initForm (UInt32 src, UInt32 dest)
{
	currencyStructure cs;
	currency_db->getUnpackedRecord (src, &cs);
	_src_currency_label = cs.code;
	currency_db->getUnpackedRecord (dest, &cs);
	_dest_currency_label = cs.code;

	_src_currency = src;
	_dest_currency = dest;

	if(results.src_amount<0) {
		results.src_amount = -results.src_amount;
		results.is_src_negative = false;
	}
	if(results.dest_amount<0) {
		results.dest_amount = -results.dest_amount;
		results.is_dest_negative = false;
	}	
}
	
Boolean EditRateForm::checkData()
{
	//should check rate format
	if (GetTextPtr (EDIT_RATE_RATE) == NULL) {
		MemHandle str_handle = DmGetResource (strRsc,RATE_IS_0_STR);
		FrmCustomAlert(GENERIC_INFO, reinterpret_cast<char*> (MemHandleLock (str_handle)), NULL, NULL);
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
		return false;
	}
	
	FlpCompDouble rate;
	FlpBufferAToF (&rate.fd, GetTextPtr (EDIT_RATE_RATE));
	if (FlpIsZero (rate.d)) {
		MemHandle str_handle = DmGetResource (strRsc,RATE_IS_0_STR);
		FrmCustomAlert(GENERIC_INFO, reinterpret_cast<char*> (MemHandleLock (str_handle)), NULL, NULL);
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
		return false;
	}
	results.exchange_rate = rate.d;

	return true;
}

void EditRateForm::saveData()
{
	//rate is saved by checkData
	results.src_amount = String2Pence(reinterpret_cast<Char*> (GetTextPtr(EDIT_RATE_AMOUNT1)));
	results.dest_amount = String2Pence(reinterpret_cast<Char*> (GetTextPtr(EDIT_RATE_AMOUNT2)));
	results.dirty = true;
}

Boolean EditRateForm::handleOpenEvent()
{
	Char* s;

	if (!osVerGe(palmos_3_5)) FrmEraseForm(frm);
	FrmDrawForm(frm);

	s=Pence2String(results.src_amount);
	SetFieldTextFromStr(EDIT_RATE_AMOUNT1, s);
	//free memory allocated by Pence2String
	MemPtrFree (s);

	s=Pence2String(results.dest_amount);
	SetFieldTextFromStr(EDIT_RATE_AMOUNT2, s);
	//free memory allocated by Pence2String
	MemPtrFree (s);

	//if we are given an exchange rate of 0, use the default exchange rate
	if (FlpIsZero (results.exchange_rate)) {
		results.exchange_rate = currency_db->getExchangeRate (_src_currency, _dest_currency);
	}
	
	FlpCompDouble rate;
	
	rate.d = results.exchange_rate;
	FlpFToA (rate.fd, _rate_buffer);
	SetFieldTextFromStr (EDIT_RATE_RATE, _rate_buffer);

	FrmCopyLabel (frm, EDIT_RATE_CUR1, _src_currency_label);
	FrmShowObject (frm, getObjectIndex (EDIT_RATE_CUR1));
	FrmCopyLabel (frm, EDIT_RATE_CUR2, _dest_currency_label);
	FrmShowObject (frm, getObjectIndex (EDIT_RATE_CUR2));

 	if (_src_currency == preferences.get()->def_currency || _dest_currency == preferences.get()->def_currency) {
		FrmShowObject (frm, getObjectIndex (EDIT_RATE_RECORD));
	}

	return true;
}
Boolean EditRateForm::handleCtlSelectEvent(UInt16 control_id)
{
	switch(control_id) {
		case EDIT_RATE_OK:
			if (checkData()) {
				saveData();
				if(results.is_src_negative) 
					results.src_amount *= -1;
				if(results.is_dest_negative) 
					results.dest_amount *= -1;
				FrmUpdateForm(caller_form_id, 0);
				FrmReturnToForm(0);
			}
			break;
		case EDIT_RATE_CANCEL:
			results.dirty = false;
			FrmReturnToForm(0);
			break;
		case EDIT_RATE_RECORD: 
			if (!checkData()) break;
			currency_db->adjustExchangeRate (_src_currency, _dest_currency, results.exchange_rate);
			break;
		case EDIT_RATE_CALC_SRC: {
			if(checkData()) {
				saveData();
			} else break;
			
			results.src_amount = (Int32)((double) (results.dest_amount) / results.exchange_rate);
			char* s=Pence2String(results.src_amount);
			SetFieldTextFromStr(EDIT_RATE_AMOUNT1, s);
			//free memory allocated by Pence2String
			MemPtrFree (s);
			break;
		}
		case EDIT_RATE_CALC_DEST: {
			if(checkData()) {
				saveData();
			} else break;
			
			results.dest_amount = (Int32)((double) (results.src_amount) * results.exchange_rate);
			char* s=Pence2String(results.dest_amount);
			SetFieldTextFromStr(EDIT_RATE_AMOUNT2, s);
			//free memory allocated by Pence2String
			MemPtrFree (s);
			break;
		}
		case EDIT_RATE_CALC_RATE: {
			if(checkData()) {
				saveData();
			} else break;
			
			results.exchange_rate = (double) results.dest_amount / (double) results.src_amount;

			FlpCompDouble rate;			
			rate.d = results.exchange_rate;
			FlpFToA (rate.fd, _rate_buffer);
			SetFieldTextFromStr (EDIT_RATE_RATE, _rate_buffer);
			break;			
		}
		default:
			return false;
	}
	return true;
}
