/**	$Revision: 1.12 $
	Last updated: $Date: 2002/08/13 16:09:29 $
	
	Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "EditScheduledTransForm.h"
#include "EditTransactionForm.h"
#include "MiscFunc.h"
#include "FreeCoinsRsc.h"

EditScheduledTransForm::EditScheduledTransForm() : Form (EDIT_SCHED_TRANS_FORM)
{}

void EditScheduledTransForm::setRecordIsNew(Boolean b)
{ 
   record_is_new = b; 
}

void EditScheduledTransForm::newSession(scheduledStructure* s, DateType* trans_date)
{ 
   st = s;
   setSaved(false);
   setRecordIsNew(true);
   start_date = trans_date;
}

Boolean EditScheduledTransForm::save()
{	
   st->frequency=StrAToI((Char*) GetTextPtr(SCHED_FREQ));
   if (!st->frequency) {
	   MemHandle str_handle = DmGetResource (strRsc,FREQUENCY_STR);
	   FrmCustomAlert(ZERO_FIELD,reinterpret_cast<char*> (MemHandleLock (str_handle)),NULL,NULL);
	   return false; //frequency has to be != 0
   }
	
   if(!st->end_by_date && st->repeat_times!=-1)
      st->repeat_times=StrAToI((Char*) GetTextPtr(SCHED_REPEAT));
   
   setSaved(true);
   setRecordIsNew(false);
   		
   return true;
}

void EditScheduledTransForm::setInterval(Repeat_interval interval)
{
	//memorise the new interval first
	st->repeat_interval = interval;
	MemHandle str_handle=NULL;
	//now display changes
	switch (interval) {
		case rep_day:
			str_handle = DmGetResource (strRsc,DAY_STR);
			FrmSetControlGroupSelection(frm,1,SCHED_OPT_DAY);
			FrmCopyLabel(frm,SCHED_INTERVAL_LABEL,reinterpret_cast<char*> (MemHandleLock (str_handle)));
			break;
		case rep_week:
			str_handle = DmGetResource (strRsc,WEEK_STR);
			FrmSetControlGroupSelection(frm,1,SCHED_OPT_WEEK);
			FrmCopyLabel(frm,SCHED_INTERVAL_LABEL,reinterpret_cast<char*> (MemHandleLock (str_handle)));
			break;
		case rep_month:
			str_handle = DmGetResource (strRsc,MONTH_STR);	
			FrmSetControlGroupSelection(frm,1,SCHED_OPT_MONTH);
			FrmCopyLabel(frm,SCHED_INTERVAL_LABEL,reinterpret_cast<char*> (MemHandleLock (str_handle)));
			break;
		case rep_year:
			str_handle = DmGetResource (strRsc,YEAR_STR);
			FrmSetControlGroupSelection(frm,1,SCHED_OPT_YEAR);
			FrmCopyLabel(frm,SCHED_INTERVAL_LABEL,reinterpret_cast<char*> (MemHandleLock (str_handle)));
			break;
		}			
	MemHandleUnlock (str_handle);
	DmReleaseResource (str_handle);
}

void EditScheduledTransForm::setDefaults()
{
	if(record_is_new) {
		//show the start date fields
		MemHandle str_handle = DmGetResource (strRsc,NEW_SCHED_STR);
		FrmCopyTitle(frm,reinterpret_cast<char*> (MemHandleLock (str_handle)) );
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);

		//show the start date fields
        FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_FIRST_TRANS_LABEL));			
        FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_START_DATE));
      
		setInterval(rep_month); //default interval is month
		
		SetFieldTextFromStr(SCHED_FREQ,"1"); //default frequency is 1
		st->frequency=1;
		
		FrmSetControlGroupSelection(frm,2,SCHED_OPT_NEVER); //default end: repeat forever
		st->repeat_times=-1;
		FrmSetControlGroupSelection (frm,3,SCHED_OPT_RECORDING); //record transactions by default	
		updateDateField(SCHED_START_DATE,*start_date,start_date_txt);						
	} else {
		if(! EditTransactionForm::instance()->getRecordIsNew())
			FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_RECORD_NOW));
        FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_NEXT_TRANS_LABEL));			
        FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_NEXT_DATE));
		MemHandle str_handle = DmGetResource (strRsc,SCHED_DETAIL_STR);
        FrmCopyTitle(frm, reinterpret_cast<char*> (MemHandleLock (str_handle)));
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
      
		setInterval(st->repeat_interval); //set the interval from the record
		SetFieldTextFromInt(SCHED_FREQ,st->frequency); //default frequency is 1
		updateDateField(SCHED_NEXT_DATE, *start_date,start_date_txt);						
		
		//now set the end conditions:
		if( st->repeat_times==-1 ) 
			FrmSetControlGroupSelection(frm,2,SCHED_OPT_NEVER); //default end: repeat forever
		else	
         if (st->end_by_date) {
				FrmSetControlGroupSelection(frm,2,SCHED_OPT_ON); //stop on a date
	        	FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_END_DATE));	
				updateDateField(SCHED_END_DATE,st->end_date,end_date_txt);
			} else { //set number of repeats
				FrmSetControlGroupSelection(frm,2,SCHED_OPT_AFTER); //stop after a no of times
	         	FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_REPEAT));			
	        	FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_REPEAT_LABEL));			
				SetFieldTextFromInt(SCHED_REPEAT,st->repeat_times); 			
			}         
		if(st->paused)
			FrmSetControlGroupSelection (frm,3,SCHED_OPT_PAUSED);
		else FrmSetControlGroupSelection (frm,3,SCHED_OPT_RECORDING);
	}
}

Boolean EditScheduledTransForm::handleOpenEvent()
{
	FrmDrawForm(frm);
	setDefaults();	
	return true;
}

void EditScheduledTransForm::recordNow()
{
	//recordnow button is not shown unless transaction has been saved

	//ask whether to alter next date
	MemHandle str_handle = DmGetResource (strRsc,WANT_ALTER_DATE_STR);
	bool rez = FrmCustomAlert(GENERIC_YES_NO,reinterpret_cast<char*> (MemHandleLock (str_handle)) , NULL, NULL);
	MemHandleUnlock (str_handle);
	DmReleaseResource (str_handle);
	
	if(rez==0) {
		//need to calculate next date
		if(transactions_db->calculateNextDate (transactions_db->getSelectedRecord())) {
			//record has been deleted; we shall need to close this form and the previous one
			MemHandle str_handle = DmGetResource (strRsc,SCHED_TRANS_DELETED_STR);
			FrmCustomAlert(GENERIC_INFO,reinterpret_cast<char*> (MemHandleLock (str_handle)) , NULL, NULL);
			MemHandleUnlock (str_handle);
			DmReleaseResource (str_handle);

			FrmUpdateForm (TRANSACTIONS_FORM, 0);
			FrmEraseForm (FrmGetFormPtr (caller_form_id));
			FrmDeleteForm (FrmGetFormPtr (caller_form_id));
			
			FrmReturnToForm(TRANSACTIONS_FORM);			
			return;
		}
		
		//now we need to update next date on screen and in our record
		transactionStructure ts;
		transactions_db->getUnpackedRecord (&ts);
		*start_date = ts.date;
		updateDateField(SCHED_NEXT_DATE, *start_date,start_date_txt);		
	}
	//record the scheduled transaction (date set to today)
	UInt32 selected = transactions_db->getSelectedUid();
	transactions_db->recordScheduledRecord(selected,true);

	//as the action above changes the selected record, re-select the current transaction
	transactions_db->selectRecord (selected);
}


Boolean EditScheduledTransForm::handleCtlSelectEvent(UInt16 control_id)
{
	switch(control_id) {
		case SCHED_TRANS_OK:
			//if something went wrong do nothing
			if(! save()) break;
			//now return to the previous form
			FrmUpdateForm(caller_form_id,0);				
			FrmReturnToForm(0);
			break; 
		case SCHED_START_DATE:
			selectDate(*start_date,SCHED_START_DATE,start_date_txt);
			break;
		case SCHED_END_DATE:
			selectDate(st->end_date,SCHED_END_DATE,end_date_txt);
			break;
		case SCHED_OPT_DAY: 
			setInterval(rep_day);
			break;
		case SCHED_OPT_WEEK: 
			setInterval(rep_week);
			break;
		case SCHED_OPT_MONTH: 
			setInterval(rep_month);
			break;
		case SCHED_OPT_YEAR: 
			setInterval(rep_year);
			break;							
		case SCHED_OPT_NEVER:
			FrmSetControlGroupSelection(frm,2,SCHED_OPT_NEVER); 
			st->repeat_times=-1;
			FrmHideObject(frm, FrmGetObjectIndex(frm, SCHED_REPEAT));			
			FrmHideObject(frm, FrmGetObjectIndex(frm, SCHED_REPEAT_LABEL));			
			FrmHideObject(frm, FrmGetObjectIndex(frm, SCHED_END_DATE));			
			st->end_by_date=false;
			break;
		case SCHED_OPT_AFTER:
			FrmSetControlGroupSelection(frm,2,SCHED_OPT_AFTER); 
			if(record_is_new || st->repeat_times<1) {
				st->repeat_times=3;
				SetFieldTextFromStr(SCHED_REPEAT,"3");
			} else {
				SetFieldTextFromInt(SCHED_REPEAT,st->repeat_times);
			}
			st->end_by_date=false;
			FrmHideObject(frm, FrmGetObjectIndex(frm, SCHED_END_DATE));			
			FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_REPEAT));			
			FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_REPEAT_LABEL));			
			break;
		case SCHED_OPT_ON:
			FrmSetControlGroupSelection(frm,2,SCHED_OPT_ON);
			if(st->repeat_times<0) st->repeat_times=0; 
			if(record_is_new || !st->end_by_date)
				DateSecondsToDate(TimGetSeconds(),&st->end_date);
			FrmHideObject(frm, FrmGetObjectIndex(frm, SCHED_REPEAT));			
			FrmHideObject(frm, FrmGetObjectIndex(frm, SCHED_REPEAT_LABEL));							
			FrmShowObject(frm, FrmGetObjectIndex(frm, SCHED_END_DATE));
			st->end_by_date=true;
			updateDateField(SCHED_END_DATE,st->end_date,end_date_txt);								
			break;
		case SCHED_OPT_PAUSED:
			FrmSetControlGroupSelection (frm,3,SCHED_OPT_PAUSED);
			st->paused = true;
			break;
		case SCHED_OPT_RECORDING:
			FrmSetControlGroupSelection (frm, 3,SCHED_OPT_RECORDING);
			st->paused = false;
			break;
		case SCHED_TRANS_CANCEL:
			//cancel button was pressed;
			//simply return to the previous form (i.e. 0)
			FrmUpdateForm(caller_form_id,0);
			FrmReturnToForm(0);
			break;
		case SCHED_RECORD_NOW:
			recordNow();
			break;
		default:
			return false;
	};
	
	return true;
}
