/**	$Revision: 1.47 $
	Last updated: $Date: 2003/01/30 00:27:43 $
	
	Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <Libraries/PalmOSGlue/TxtGlue.h>
#include "EditTransactionForm.h"

#include "SplitsDB.h"
#include "PreferencesForm.h"
#include "CoinsPref.h"
#include "ChooseAccountForm.h"
#include "FreeCoinsRsc.h"
#include "AccountsDB.h"
#include "FormManager.h"
#include "TransactionsForm.h"

#include <StdIOPalm.h>
EditTransactionForm* EditTransactionForm::_instance = NULL;
EditTransactionForm edit_trans_form;

EditTransactionForm::EditTransactionForm () : Form (EDIT_TRANS_FORM)
{
	_note_form = NoteForm::instance();
	_splits_form = SplitsForm::instance();
	_calculator_form = CalculatorForm::instance();
	_edit_rate_form = new EditRateForm();
}

EditTransactionForm::~EditTransactionForm()
{
	delete _edit_rate_form;
	FormManager::instance()->deregisterForm (form_id);
}


EditTransactionForm* EditTransactionForm::instance()
{
	if(_instance == NULL) {
		_instance = &edit_trans_form;
	}
	return _instance;
}

Boolean EditTransactionForm::checkData()
{
	Char *s;
	UInt8 i;
	Int32 split_total = 0;

	saveToSplits();

	// check the amount
	s = reinterpret_cast<Char*>(GetTextPtr(EDIT_TRANS_AMMOUNT));
	if (!s || !validNumberStr(s, false) || _splits_form->getAmount(_this_split)==0) {
		MemHandle str_handle = DmGetResource (strRsc,POZ_NUMBER_STR);
		FrmCustomAlert(GENERIC_ERROR,reinterpret_cast<char*> (MemHandleLock (str_handle)), NULL, NULL);
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
		return false;
	}

	// check if the transaction is balanced
	if(_show_rate) {
		if (_this_split < _peer_split) split_total = (Int32)( _splits_form->getAmount (_this_split) * t.exchange_rate) +  _splits_form->getAmount (_peer_split);
		else split_total = (Int32)( _splits_form->getAmount (_peer_split) * t.exchange_rate) +  _splits_form->getAmount (_this_split);
	} else {
		for (i=0; i<_splits_form->getCount(); i++)
			split_total += _splits_form->getAmount(i);
		t.exchange_rate = 1.0;
	}
	
	if (split_total!=0) {
		MemHandle str_handle = DmGetResource (strRsc,UNBALANCED_STR);
		bool rez=FrmCustomAlert(GENERIC_YES_NO,reinterpret_cast<char*> (MemHandleLock (str_handle)),NULL, NULL);
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
		if(rez) return false;
	}

	return true;
}

void EditTransactionForm::save()
{	
	splitStructure spl;
	Boolean cleared;
	UInt32 trans_uid;
	UInt8 index;
	MemHandle payee_handle = NULL;
	
	t.num_splits = _splits_form->getCount();

	t.payee=reinterpret_cast<Char*>(GetTextPtr(EDIT_TRANS_PAYEE));
	if(!t.payee) {
		payee_handle = DmGetResource (strRsc,NA_STR);
		t.payee=reinterpret_cast<char*> (MemHandleLock (payee_handle));

	}

	t.num = reinterpret_cast<Char*>(GetTextPtr(EDIT_TRANS_NUM));
	//leaving NULLs in database records will result in errors during packing
	if(!t.num) t.num = "";
	if(!t.note) t.note = "";

    
	t.method=(Payment_method)LstGetSelection((ListPtr)GetObjectPtr(EDIT_TRANS_TYPE_LIST));

    if (!_record_is_new) {
		transactions_db->deleteRecord(transactions_db->getSelectedRecord());
	}

	//this sets the cleared state and creates the new record. for scheduled records the cleared field will
	//indicate whether the record will be cleared or not when introduced
	cleared = FrmGetControlValue(frm, getObjectIndex(EDIT_TRANS_CLEARED_CBX));
	UInt16 trans_id = transactions_db->newRecord(&t,cleared,_trans_is_sched);
	trans_uid = transactions_db->getRecordUid(trans_id);	

	spl.trans_uid = trans_uid;
	for (index = 0; index<t.num_splits; index++) {
		spl.line_no = index + 1;
		spl.acc_uid = _splits_form->getAccount(index);
		spl.amount = _splits_form->getAmount(index);
		splits_db->newRecord(&spl, cleared);
	}

	saved=true;
	if(payee_handle) {
		MemHandleUnlock (payee_handle);
		DmReleaseResource (payee_handle);
	}
}

void EditTransactionForm::setDefaults()
{
	ControlPtr ctl;
	Int32 amount;
	UInt32 trans_uid = 0;
	UInt16 count, index;
	UInt8 split_index;
	
	AccountType type;
	accountStructure as;
	splitStructure spl;

	accounts_db->getUnpackedRecord(_prev_acc_uid, &as);
	type = as.type;

	//set debit/credit button labels
 	SysStringByIndex(ACC_DEBIT_DESC, (UInt8)type, _debit_label, 32);
	ctl = reinterpret_cast<ControlPtr>(GetObjectPtr(EDIT_TRANS_DEBIT));
	CtlSetLabel(ctl, _debit_label);
	SysStringByIndex(ACC_CREDIT_DESC, (UInt8)type, _credit_label, 32);
	ctl = reinterpret_cast<ControlPtr>(GetObjectPtr(EDIT_TRANS_CREDIT));
	CtlSetLabel(ctl, _credit_label);
   
	//let the scheduled form know that we started a new session
	_edit_scheduled_trans_form.newSession(&t.sched, &t.date);

    //no matched payee
    StrCopy (_matched_payee,"");
    StrCopy (_last_unmatched_payee,"");

	if (!_record_is_new) { //we are editing an existing record
		transactions_db->getUnpackedRecord(&t);
		//get the uid and amount
		trans_uid = transactions_db->getSelectedUid();
		
		amount = transactions_db->getAmount(trans_uid, _prev_acc_uid);
		_is_debit = (amount>0);

		count = t.num_splits;	
		_splits_form->setCount(count);
		split_index = 0;
		for (index=splits_db->locateFirstSplit(trans_uid);count>0;index++,split_index++,count--) {
				splits_db->getUnpackedRecord(index, &spl);
				_splits_form->setAccount(split_index, spl.acc_uid);
				_splits_form->setAmount(split_index, spl.amount);
				if (spl.acc_uid == _prev_acc_uid)
					_this_split = split_index;
				else {
					_peer_split = split_index;
					_peer_acc = spl.acc_uid;
				}
			}

		//set cleared status
		if(transactions_db->isCleared(transactions_db->getSelectedRecord()))
			FrmSetControlValue(frm,FrmGetObjectIndex(frm,EDIT_TRANS_CLEARED_CBX),1);
		else 
			FrmSetControlValue(frm,FrmGetObjectIndex(frm,EDIT_TRANS_CLEARED_CBX),0);

		//date
		updateDateField(EDIT_TRANS_CHOOSE_DATE,t.date,_date_txt);				

		//now we can fill in the text fields
		SetFieldTextFromStr(EDIT_TRANS_PAYEE,t.payee);
		SetFieldTextFromStr(EDIT_TRANS_NUM,t.num);
      
		//if our record is not new and was marked as scheduled we can edit its scheduled information
		if(transactions_db->isScheduled(transactions_db->getSelectedRecord())) {
			_edit_scheduled_trans_form.setRecordIsNew(false);
			_trans_is_sched = true; //let this form know that the transaction is scheduled
		}

		FrmShowObject(frm, getObjectIndex(EDIT_TRANS_DELETE));
		MemHandle str_handle = DmGetResource (strRsc,TRANS_DETAIL_STR);
		FrmCopyTitle(frm,reinterpret_cast<char*> (MemHandleLock (str_handle)) );
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
		saved=true; //record was saved already in a prev session		

		_show_rate = false;		
		if (_splits_form->getCount()==2) {
			if (accounts_db->getAccountCurrency (_prev_acc_uid) != accounts_db->getAccountCurrency (_peer_acc))
				_show_rate = true;
		}

		if (_show_rate) FrmShowObject (frm, getObjectIndex (EDIT_TRANS_CURRENCY));

	} else { //record is new
		amount = 0;
		_is_debit = (type==EXPENSE);

		_splits_form->setCount(1);
		_splits_form->setAccount(0, _prev_acc_uid);
		_splits_form->setAmount(0, 0);
		t.exchange_rate = 0.0;

		DateSecondsToDate(TimGetSeconds(),&t.date);
		t.payee = "";
		t.num = "";
		t.note = "";
		_peer_acc = 0;
		t.num_splits = 1;
		t.method = preferences.get()->payment_method;
		MemHandle str_handle = DmGetResource (strRsc,NOT_SET_STR);
		CtlSetLabel(reinterpret_cast<ControlPtr>(GetObjectPtr(EDIT_TRANS_CHOOSE_ACC)),
					reinterpret_cast<char*> (MemHandleLock (str_handle)));
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
		FrmSetControlValue(frm,getObjectIndex(EDIT_TRANS_CLEARED_CBX),preferences.get()->trans_cleared);		
		updateDateField(EDIT_TRANS_CHOOSE_DATE,t.date,_date_txt);				
		str_handle = DmGetResource (strRsc,NEW_TRANS_STR);
		FrmCopyTitle(frm,reinterpret_cast<char*> (MemHandleLock (str_handle)) );
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
		saved=false; //only starting; didnt save anything yet
		// update peer account and amount
	}
	// update peer account and amount
	readFromSplits();

	//fill in and select the right element in the transaction type list  
	PopupListSetSelection(EDIT_TRANS_TYPE_LIST, EDIT_TRANS_TYPE_TRIGGER,(Int16)t.method);
}

void EditTransactionForm::readFromSplits()
{
	/* the following codes are used: 
	   0 - unbalanced one split 
	   1 - balanced transaction with 2 
	   2 - all other transactions */
	Int32 splits_total = 0;
	Int32 this_amount = 0;
	Char *s;

	t.num_splits = _splits_form->getCount();
	
	for (UInt8 index=0; index<t.num_splits; index++) {
		splits_total += _splits_form->getAmount(index); 
		if (_splits_form->getAccount(index)==_prev_acc_uid) {
			_this_split = index;
			this_amount = _splits_form->getAmount(index);
		} else {
			_peer_split = index;
			_peer_acc = _splits_form->getAccount(index);
		}
	}
	/* we need to handle the currency excange case: we have at most 2 splits
	   if 1st split is our account split, we have a forward transaction, otherwise a reverse transaction 
	   the second check acts as a safeguard - the _show_rate varialble may have not been updated at the time
	   this is being executed*/
	if(_show_rate && t.num_splits == 2) { 
		if (_this_split < _peer_split)  //forward transaction
			splits_total = (Int32)( _splits_form->getAmount (_this_split) * t.exchange_rate) +  _splits_form->getAmount (_peer_split);
		else //reverse transaction
			splits_total = (Int32)( _splits_form->getAmount (_peer_split) * t.exchange_rate) +  _splits_form->getAmount (_this_split);
	}

	if (splits_total==0 && t.num_splits==2)
		_trans_class = 1;
	else if (t.num_splits==1) _trans_class = 0;
	else _trans_class = 2;		

	MemHandle str_handle = NULL;
	switch (_trans_class) {
		case 1:
			accounts_db->getFullNameTruncLeftToFit(_peer_acc, 120, _acc_name);	
			break;
		case 0:
			str_handle = DmGetResource (strRsc,NOT_SET_STR);
			StrCopy(_acc_name,reinterpret_cast<char*> (MemHandleLock (str_handle)) );
			_peer_split = _peer_acc = 0;
			break;
		case 2:
			str_handle = DmGetResource (strRsc,D_SPLIT_D_STR);
			StrCopy(_acc_name, reinterpret_cast<char*> (MemHandleLock (str_handle)));
			break;
	}
	if(str_handle) {
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
	}
	CtlSetLabel(reinterpret_cast<ControlPtr>(GetObjectPtr(EDIT_TRANS_CHOOSE_ACC)),_acc_name);

	//select the right radio button
	if (this_amount>0)
		_is_debit = true;
	if (this_amount<0)
		_is_debit = false;
	if (_is_debit)
		FrmSetControlGroupSelection(frm,1,EDIT_TRANS_DEBIT);
	else
		FrmSetControlGroupSelection(frm,1,EDIT_TRANS_CREDIT);

	if (this_amount != 0) {
		s = Pence2String(_is_debit?this_amount:-this_amount);
		SetFieldTextFromStr(EDIT_TRANS_AMMOUNT,s);
		MemPtrFree(s); 
	}
}

void EditTransactionForm::saveToSplits(void)
{
	Int32 amount;

	amount = String2Pence(reinterpret_cast<Char*>(GetTextPtr(EDIT_TRANS_AMMOUNT)));
	if (!_is_debit) amount *= -1;

	_splits_form->setAmount(_this_split, amount);
	if (_trans_class==1) {
		_splits_form->setAccount(_peer_split, _peer_acc);
	
        if(! _show_rate) {
            _splits_form->setAmount(_peer_split, -amount);
        } else {
            double rate = t.exchange_rate;
            if (_this_split > _peer_split) rate = 1.0/rate;
            _splits_form->setAmount(_peer_split, -(Int32)(amount*rate));
        }
    }
}

bool EditTransactionForm::handleAutocompletion()
{
    char* payee = reinterpret_cast<char*>(GetTextPtr(EDIT_TRANS_PAYEE));
    int orig_len = StrLen(payee);
    
    if(orig_len<2) return false; //match from 2 chars upwards
    
    //if matched already, return (handle case when either string is a substring of the other one)
    if (StrNCaselessCompare (payee,_matched_payee,orig_len)==0
           && StrLen (_matched_payee)>0
           && orig_len<=StrLen (_matched_payee)) {
        SetFieldTextFromStr(EDIT_TRANS_PAYEE,_matched_payee);
        FldSetSelection(reinterpret_cast<FieldType*>(GetObjectPtr(EDIT_TRANS_PAYEE)), orig_len, StrLen(_matched_payee));
        return false;
    }

    //if unmatched already, return (handle case when either string is a substring of the other one)
    if (StrNCaselessCompare (payee,_last_unmatched_payee,StrLen (_last_unmatched_payee)) == 0
           && StrLen (_last_unmatched_payee)>0
           && orig_len>=StrLen (_last_unmatched_payee)) {
        return false;
    }
    
    Int32 rez = transactions_db->getTransactionByPayeeAndAcc(payee,0); //this may be a problem if UID>maxInt32
    if (rez>=0) { //found match for this account
        UInt32 tr_uid = (UInt32) rez;
        transactionStructure ts;
        transactions_db->getUnpackedRecord(tr_uid, &ts);
        //if the text is exactly the same do not update it again (prevents things like cursor movements continously causing updates)
        if (StrCaselessCompare(payee, ts.payee) == 0 && StrLen(ts.payee) != 1) return false;
				
        SetFieldTextFromStr(EDIT_TRANS_PAYEE,ts.payee);
        FldSetSelection(reinterpret_cast<FieldType*>(GetObjectPtr(EDIT_TRANS_PAYEE)), orig_len, StrLen(ts.payee));					
        StrCopy (_matched_payee,ts.payee); //save match
        StrCopy (_last_unmatched_payee,"");
        
        //copy transaction method
        PopupListSetSelection(EDIT_TRANS_TYPE_LIST, EDIT_TRANS_TYPE_TRIGGER, (Int16) ts.method);					
        t.exchange_rate = ts.exchange_rate;
        _splits_form->deleteAllSplits();
        //copy splits
        splitStructure spl;
        _splits_form->setCount(ts.num_splits);
        for (UInt16 index=splits_db->locateFirstSplit(tr_uid), split_index = 0; 
             split_index<ts.num_splits;
             index++,split_index++) {
            MemHandle rec_h = splits_db->getUnpackedRecord(index, &spl);
            ErrFatalDisplayIf (!rec_h,"Failed to get unpacked split");
            _splits_form->setAccount(split_index, spl.acc_uid);
            _splits_form->setAmount(split_index, spl.amount);
        }
        readFromSplits();
    } else { //cant match  account. try match just payee text
        rez = transactions_db->getTransactionByPayee(payee,0);
        if (rez>=0) { //so there is some transaction with a payee which starts the same as the text we have
            UInt32 tr_uid = (UInt32) rez;
            int orig_len = StrLen(payee);
            transactionStructure ts;
            transactions_db->getUnpackedRecord(tr_uid, &ts);
            if (StrCaselessCompare(payee, ts.payee) == 0 && StrLen(ts.payee) != 1) return false;
				
            SetFieldTextFromStr(EDIT_TRANS_PAYEE,ts.payee);
            StrCopy (_matched_payee,ts.payee); //save match
            StrCopy (_last_unmatched_payee,"");
            
            FldSetSelection(reinterpret_cast<FieldType*>(GetObjectPtr(EDIT_TRANS_PAYEE)), orig_len, StrLen(ts.payee));					
            //copy transaction method
            PopupListSetSelection(EDIT_TRANS_TYPE_LIST, EDIT_TRANS_TYPE_TRIGGER, (Int16) ts.method);					
        } else { //no match
            StrCopy (_matched_payee,"");
            if(orig_len>=2 && StrLen (_last_unmatched_payee) == 0)
                StrCopy (_last_unmatched_payee,payee);
            return false;
        }
    }
    return true;
}



Boolean EditTransactionForm::handleUpdateEvent (UInt16 update_code)
{	
	if (update_code == 5) {
		//handle note update case
		if(_note_form->updatedData (form_id)) {
			t.note = _note_form->getNote();
		}
		return true;
	}	

	//handle return from Exchange rate edit
	if(_edit_rate_form->results.dirty){
		if (_this_split > _peer_split) 
			t.exchange_rate = 1.0/_edit_rate_form->results.exchange_rate;
		else t.exchange_rate = _edit_rate_form->results.exchange_rate;
		
		_splits_form->setAmount(_this_split, _edit_rate_form->results.src_amount);
		_edit_rate_form->results.dirty = false;
		_is_debit = (_edit_rate_form->results.src_amount<0);
		
		char* s = Pence2String(_is_debit?-_edit_rate_form->results.src_amount:_edit_rate_form->results.src_amount);	
		SetFieldTextFromStr(EDIT_TRANS_AMMOUNT,s);
		MemPtrFree(s); 

		saveToSplits();
		readFromSplits();
	}
	
	/* deal with update of the payee field (autocompletion trigger) */
	if(update_code == 1) 
        if(!handleAutocompletion()) return true;

    //handle update when coming back from chosing an account
	if(ChooseAccountForm::instance()->chosen) {
		ChooseAccountForm::instance()->chosen = false;
		_peer_acc = ChooseAccountForm::instance()->chosen_account;
		if (_trans_class==0) {
			_splits_form->setCount(2);
			_peer_split = 1;
			_trans_class = 1;
		}
		saveToSplits();
		readFromSplits();
	}	

	//update the date field (could have been changed by editscheduledtransform)
	updateDateField(EDIT_TRANS_CHOOSE_DATE,t.date,_date_txt);		
	if(_edit_scheduled_trans_form.getSaved()) //if user saved the scheduled form, this trans is scheduled
		_trans_is_sched = true;

	//something updated in the splits
	if (update_code == 2)
		readFromSplits();

	//handle data received from the calculator
	if (update_code == _calculator_form->update_code) {
		Int32 amount = _calculator_form->amount*_calculator_form->sign;
		if(amount<0) { 
			_is_debit = true;
			FrmSetControlGroupSelection(frm,1,EDIT_TRANS_DEBIT);
		} else {
			_is_debit = false;
			FrmSetControlGroupSelection(frm,1,EDIT_TRANS_CREDIT);
		}
		
		char* s = Pence2String(_is_debit?amount:-amount);	
		SetFieldTextFromStr(EDIT_TRANS_AMMOUNT,s);
		MemPtrFree(s); 
	
		saveToSplits();
		readFromSplits();
	}

    //handle multiple currencies (display/hide the rate button)
    _show_rate = false;
	
	if (_splits_form->getCount()==2) {
		if (accounts_db->getAccountCurrency (_prev_acc_uid) != accounts_db->getAccountCurrency (_peer_acc))
			_show_rate = true;
	}
	if (_show_rate) FrmShowObject (frm, getObjectIndex (EDIT_TRANS_CURRENCY));
	else FrmHideObject (frm, getObjectIndex (EDIT_TRANS_CURRENCY));
		
		
	if (!osVerGe(palmos_3_5)) FrmEraseForm(frm);
	FrmDrawForm(frm);
	return true;
}

Boolean EditTransactionForm::handleCloseEvent()
{	
	_splits_form->deleteAllSplits();
	return Form::handleCloseEvent();
}

Boolean EditTransactionForm::handleOpenEvent()
{
	// remember what account we were in when we started this form
	_prev_acc_uid = accounts_db->getSelectedUid();

	//draw form
	FrmDrawForm(frm);
	setDefaults(); //this will initialise the scheduled form
	
	FrmSetFocus(frm, FrmGetObjectIndex(frm,EDIT_TRANS_PAYEE));
	/**	note that no transasction can be scheduled by default. A transaction is scheduled when:
		1. its saved and it's category is set to scheduled
		2. its scheduling information was edited and saved (no matter whether the tranasction itself is new or old)   	*/
	return true;
}

Boolean EditTransactionForm::handleCtlSelectEvent (UInt16 control_id)
{
	switch(control_id) {
		case EDIT_TRANS_OK:
			//if something is wrong, an error message has already been
			//issued by checkData, so do nothing.            
			if(!checkData())
				break;
			save();

			if(_trans_is_sched) {
				_trans_is_sched=false; //set the scheduled flag to false for next call
				transactions_db->schedule(true); //force the scheduling to happen now
			}

			// free the temporary splits array
			_splits_form->deleteAllSplits();
			//update the main form and return to the previous form
			FrmUpdateForm(caller_form_id,0);				

			FrmReturnToForm(0);
			break;
		case EDIT_TRANS_DEBIT: //the radio buttons
			_is_debit=true;
            saveToSplits();
            break;
		case EDIT_TRANS_CREDIT:
			_is_debit=false;
            saveToSplits();
			break;				
		case EDIT_TRANS_CHOOSE_DATE:
			selectDate(t.date,EDIT_TRANS_CHOOSE_DATE,_date_txt);
			break;
		case EDIT_TRANS_CHOOSE_ACC:
			if (_trans_class==2) {
				saveToSplits();
				popup(_splits_form);
			} else {
				ChooseAccountForm::instance()->chosen = false;
				ChooseAccountForm::instance()->chosen_account = _peer_acc;
				ChooseAccountForm::instance()->allow_new_account = true;
				popup(ChooseAccountForm::instance());		
			}
			break;
		case EDIT_TRANS_CANCEL:
			//cancel button was pressed;
			//simply return to the main form
			_trans_is_sched=false;				
			_splits_form->deleteAllSplits();
			// playing with the splits may have changed the selected acct.
			// undo that.
			accounts_db->selectRecord(_prev_acc_uid);
			FrmUpdateForm(caller_form_id,0);
			FrmReturnToForm(0);
			break;
		case EDIT_TRANS_DELETE: {
			//delete button was pressed;
			//simply return to the main form
			// We shouldn't get here if the transactions is new,
			// but just to be safe...
			if (_record_is_new) break;
			// ask for confirmation
			MemHandle str_handle = DmGetResource (strRsc,TRANSACTION_STR);
			bool rez = FrmCustomAlert(DELETE_ALERT,reinterpret_cast<char*> (MemHandleLock (str_handle)) , NULL, NULL);
			MemHandleUnlock (str_handle);
			DmReleaseResource (str_handle);
			if(rez) break;
			_trans_is_sched=false;
			transactions_db->deleteRecord(transactions_db->getSelectedRecord());
			_splits_form->deleteAllSplits();

			//the event does not get through. we have to force update
			FrmUpdateForm(caller_form_id,0);	
			TransactionsForm::instance()->handleUpdateEvent (2);
			FrmReturnToForm(0);
			break;
		}
		case EDIT_TRANS_SCHED:
			popup (&_edit_scheduled_trans_form);
			break;
		case EDIT_TRANS_RECORD_SPLITS:
			saveToSplits();
			popup(_splits_form);
			break;
		case EDIT_TRANS_CALC:
			saveToSplits();
			readFromSplits();
			_calculator_form->amount = _splits_form->getAmount(_this_split);
			if (_calculator_form->amount < 0 ||
				(_calculator_form->amount == 0 && !_is_debit)) {
				_calculator_form->amount *= -1;
				_calculator_form->sign = -1;
			} else _calculator_form->sign = 1;
			popup(_calculator_form);
			break;
		case EDIT_TRANS_NOTE:
			_note_form->initForm (t.payee, t.note);
			popup (_note_form);
			break;
		case EDIT_TRANS_CURRENCY: {
			saveToSplits();			
			_edit_rate_form->results.src_amount = _splits_form->getAmount (_this_split);
			_edit_rate_form->results.dest_amount = _splits_form->getAmount (_peer_split);
			_edit_rate_form->results.exchange_rate = (_this_split > _peer_split) ? 1.0/t.exchange_rate:t.exchange_rate;
			_edit_rate_form->initForm (accounts_db->getAccountCurrency (_prev_acc_uid), accounts_db->getAccountCurrency (_peer_acc));
			popup (_edit_rate_form);
			break;
		}
		default:
			return false;
	};
	return true;
}

Boolean EditTransactionForm::handleKeyDownEvent(WChar key_id, UInt16 modifiers)
{
	//check that the event was generated while the field is in focus; this does not guarantee that we intercept only char entry events,
	//but at least it restricts the number of events intercepted.
    //also, ignore delete events for convenience
	if (FrmGetObjectId(frm,FrmGetFocus(frm)) == EDIT_TRANS_PAYEE
		&& !TxtGlueCharIsVirtual(modifiers, key_id)
        && key_id != 8) {        
		FrmUpdateForm(form_id,1);
	}
    if (key_id == 8) {
        StrCopy (_matched_payee,"");
        StrCopy (_last_unmatched_payee,"");
    }
    
	//only sniffing the event, let the system process it properly
	return false;
}
