/**	$Revision: 1.29 $
	Last updated: $Date: 2002/12/19 22:52:47 $
	
	Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef __EDIT_TRANSACTION_FORM__
#define __EDIT_TRANSACTION_FORM__

#include <PalmOS.h>

#include "Form.h"
#include "TransactionsDB.h"
#include "EditScheduledTransForm.h"
#include "NoteForm.h"
#include "SplitsForm.h"
#include "CalcForm.h"
#include "EditRateForm.h"

///the form is used to create a new transaction or to modify an existing one, depending on the record_is_new member
class EditTransactionForm : public Form
{
public:
    //    typedef enum {autocompletion=1,splits,calculator,note} updateCodes;
    
    ///constructor
	EditTransactionForm() FORM_SECTION2;
	///destructor - disposes of allocated memory
	virtual ~EditTransactionForm() FORM_SECTION2;
	
	///returns singleton reference to this object
	static EditTransactionForm* instance() FORM_SECTION2;

	///@return value of member trans_id
	UInt16	getTransId() { return _trans_id; }
	///sets value of memeber trans_id
	void setTransId(UInt16 i) { _trans_id=i; }
	///sets value of trans_is_sched
	void setTransIsSched(Boolean b) { _trans_is_sched = b; }
	///sets value of records_is_new
	void setRecordIsNew(Boolean b) { _record_is_new = b; }
	/// @return value of record_is_new
	bool getRecordIsNew() { return _record_is_new; }

private:
	///reference to this object
	static EditTransactionForm* _instance;
	/// this is set to true if we want to create a new account and to false if we want the 'details' functionality
	Boolean _record_is_new;
	/// set to true when creating/editing a scheduled transaction
	Boolean _trans_is_sched;
	/// true if the current transaction is a debit
	Boolean	_is_debit;
	///id of the transaction being manipulated
	UInt16 _trans_id;
	///structure used to hold data entered by user
	transactionStructure t;
	//the peer account for this transaction
	UInt32 _peer_acc;
	///name of the account chosen as peer
	Char _acc_name[60];
	///date text
	Char _date_txt[dateStringLength];
	///label for debit button
	Char _debit_label[32];
	///label for credit button
	Char _credit_label[32];
    ///last matched payee; null if none
    char _matched_payee[32];
    ///last unmatched payee. used to avoid screen updates and db checks whenever possible
    char _last_unmatched_payee[32];

	/** transaction scheduling form is accessed through this object. As scheduling is only accessed from
		this form, the scheduling form is an object contained within this class */
	EditScheduledTransForm _edit_scheduled_trans_form;
	/// classify current split configuration 
	UInt8 _trans_class;
	/// index of the split that affects the current account
	UInt8 _this_split;
	/// index of the split that affects the peer account, if applicable
	UInt8 _peer_split;
	/// remember which account was selected when form was opened
	UInt32 _prev_acc_uid;

	///form to handle notes
	NoteForm* _note_form;
	///form handlind split management
	SplitsForm* _splits_form;
	///generic calculator
	CalculatorForm* _calculator_form;
	///form to handle currency conversion rates
	EditRateForm* _edit_rate_form;

	/** handle possiblity of currency exchange; Show the button accessing the currency form iff there are only
	   two accounts involved and they use different currencies */
	bool _show_rate;

	/// pre-save data check, returns true if data is good, false otherwise.
	Boolean checkData() FORM_SECTION2;
	///saves the transaction data and updates all the accounts involved
	void save() FORM_SECTION2;
	///set the values for some of the fields in the form (from the transaction record)
	void setDefaults() FORM_SECTION2;
	/// reflect peer account and transaction amount in underlying splits array
	void saveToSplits() FORM_SECTION2;
	/// update amount and peer account according to the underlying splits array
	void readFromSplits() FORM_SECTION2;
	///displays the name for the given account
	void setPeerAccount(UInt32 account) FORM_SECTION2;
	/** handle keystrokes and optionally autocomplete.
           Autocompletion follows the following steps:
           * intercept all keystrokes except virtual ones (from handleKeyDownEvent)
           Cache the following cases:
             - previously matched the input against a longer payee. If keystrokes continue to fit the previous match, do not re-check DB or update screen
             - previously could not find any payee starting with the text in the input. Any other input starting with that text is considered un-matchable

           * attempt to find a transaction from the current account matching the payee
           * if one is found, copy splits, full text and method
           * otherwise just try to match a transaction with the same payee and fill in the payee and method 
           @return true if something updated, false otherwise*/
    bool handleAutocompletion() FORM_SECTION2;

	///draws form and table
	Boolean handleOpenEvent() FORM_SECTION2;
	/// form close
	Boolean handleCloseEvent() FORM_SECTION2;
	///autocompletion hook
	Boolean handleKeyDownEvent (WChar key_id, UInt16 modifiers) FORM_SECTION2;
	///handles form controls
	Boolean handleCtlSelectEvent (UInt16 control_id) FORM_SECTION2;
	///handles update events
	Boolean handleUpdateEvent (UInt16 update_code) FORM_SECTION2;
};
#endif
