/** $Revision: 1.15 $
	Last updated: $Date: 2002/09/17 16:13:03 $
		
	Copyright (C) 2001-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "Form.h"
#include "FormManager.h"

Form::Form(UInt16 id)
{
//	ErrFatalDisplay ("form constructor");
	saved = false;
	form_id = id;
	//register form with the form manager, unless default id has been passed
	if(id) {
		//		char ermsg[50];
		//StrPrintF (ermsg,"form about to register form: %d",id);
		//ErrFatalDisplay (ermsg);
		FormManager::instance()->registerForm(id, this);
	}
}

Form::~Form()
{
	//de-register form automatically
    FormManager::instance()->deregisterForm(form_id);
}

/* first make sure this form will handle events if more than one instance, then execute popup */
void Form::popup()
{
	FormManager::instance() -> bringToFront (form_id,this);
	FrmPopupForm(form_id);
}

void Form::popup(Form* form)
{
	form->setCallerFormId (form_id);
	form->popup();
}

UInt16      Form::doDialog(FormPtr parent_form) 
{ 	
    //for dialogs we have to take care of load/close ourselves
    handleLoadEvent();
    UInt16 rez = FrmDoDialog(frm); 
    handleCloseEvent();
    FrmSetActiveForm(parent_form);
    return rez;
}

Boolean     Form::handleKeyDownEvent(WChar key_id, UInt16 modifiers)
{
    return false;
}

Boolean     Form::handleOpenEvent()
{
	//draw form
	FrmDrawForm(frm);
	return true;
}

Boolean     Form::handleCtlSelectEvent(UInt16 control_id)
{
	return false;
}

Boolean     Form::handleTblSelectEvent(tblSelect& tbl_select)
{
	return false;
}

Boolean     Form::handleLstSelectEvent(UInt16 item)
{
	return false;
}

Boolean     Form::handleMenuEvent(UInt16 menu_item_id)
{
	return false;
}

Boolean     Form::handleUpdateEvent(UInt16 update_code)
{
	return false;
}

Boolean Form::handlePenDownEvent (Int16 x_coord, Int16 y_coord)
{
	return false;
}

Boolean Form::handlePopSelect (popSelect& pop_select)
{
	return false;
}

Boolean Form::handleFldChangedEvent()
{
	return false;
}

Boolean Form::handleSclRepeatEvent(UInt16 new_value, UInt16 value)
{
	return false;
}

Boolean     Form::handleLoadEvent()
{
	frm = FrmInitForm(form_id);
	FrmSetActiveForm(frm);          
	return true;
}

Boolean     Form::handleCloseEvent()
{
	FrmEraseForm(frm);
	FrmDeleteForm(frm);
	return true;
}

Boolean Form::handleEvent(EventPtr event)
{
	Boolean handled = true;

	// Get the form pointer
	frm = FrmGetActiveForm();

	// Key Events
	switch(event->eType) {
	    // Form Open
	    case frmOpenEvent:
            handled = handleOpenEvent();
		    break;
    	case keyDownEvent:
            handled = handleKeyDownEvent(event->data.keyDown.chr,event->data.keyDown.modifiers);
	    	break;
    	// Controls
    	case ctlSelectEvent:
    		handled = handleCtlSelectEvent( event->data.ctlSelect.controlID );
    		break;
    	case tblSelectEvent: {	
			tblSelect tbl_sel;
			tbl_sel.tableID = event->data.tblSelect.tableID;
			tbl_sel.pTable = event->data.tblSelect.pTable;
			tbl_sel.row  = event->data.tblSelect.row;
			tbl_sel.column = event->data.tblSelect.column;
            handled = handleTblSelectEvent(tbl_sel);
	    	break;
		}
	    // Menus
	    case menuEvent:
            handled = handleMenuEvent(event->data.menu.itemID);
		    break;
	    // Form Close
	    case frmCloseEvent:
            handled = handleCloseEvent();
            break;
	    // Form Update
	    case frmUpdateEvent:
			//			ErrFatalDisplay ("handlind update evt");
            handled = handleUpdateEvent(event->data.frmUpdate.updateCode);
		    break;
		case popSelectEvent: {
			popSelect pop_sel;
			pop_sel.controlID = event->data.popSelect.controlID;
			pop_sel.listID = event->data.popSelect.listID;
			pop_sel.controlP = event->data.popSelect.controlP;
			pop_sel.listP = event->data.popSelect.listP;
			pop_sel.selection = event->data.popSelect.selection;
			pop_sel.priorSelection = event->data.popSelect.priorSelection;
			handled = handlePopSelect (pop_sel);
			break;
		}
		case penDownEvent:
			handled = handlePenDownEvent (event->screenX, event->screenY);
			break;
		case sclRepeatEvent:
			handled = handleSclRepeatEvent (event->data.sclRepeat.newValue, event->data.sclRepeat.value);
			break;
		case fldChangedEvent:
			handled = handleFldChangedEvent();
			break;
		case lstSelectEvent:
			handled = handleLstSelectEvent (event->data.lstSelect.selection);
			break;
	    // Everything else...
	    default:
		    handled = false;
	}	
	return handled;		
}

UInt16 Form::getObjectIndex (UInt16 object_id)
{
	return FrmGetObjectIndex (frm, object_id);
}
