/**	$Revision: 1.11 $
	Last updated: $Date: 2002/09/24 19:46:42 $
	
	Copyright (C) 2001-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "FormManager.h"

FormManager* FormManager::_instance = NULL;
Boolean FormManager::_destroy_requested = false;
UInt8 FormManager::_form_count = 0;

FormManager* FormManager::instance()
{
	if(_instance==NULL) {
		//		_instance = &a_form_manager;
		_instance = new FormManager();
	}
	return _instance;
}

void FormManager::suicide()
{
	_destroy_requested = true;
	destroyInstance();
}

void FormManager::destroyInstance()
{	
	if(_destroy_requested && _instance && _form_count == 0) {
		delete _instance; 
		_instance = NULL;
	}
}


FormManager::FormManager()
{ 
	for(int i=0;i<max_forms;i++) {
		forms[i].active=false;
    }
}

void FormManager::registerForm(UInt16 form_id, Form* form)
{
    for(int i=0;i<max_forms;i++) {
        if(! forms[i].active) {
            forms[i].active = true;
            forms[i].frm = form;
            forms[i].form_id = form_id;
			_form_count++;
            return;
        }
    }
    ErrFatalDisplay("Ran out of form slots :-(");
}

void FormManager::deregisterForm(UInt16 form_id)
{
    for(int i=0;i<max_forms;i++) {
        if(forms[i].form_id == form_id &&  forms[i].active) {
            forms[i].active = false;
			_form_count--;
			destroyInstance();
            return;
        }
    }
	//	ErrFatalDisplay("The form was not registered with the form manager");        
}

Boolean FormManager::dispatchLoadEvent(UInt16 form_id)
{
    for(int i=0;i<max_forms;i++) {
        if(forms[i].form_id == form_id && forms[i].active) {
            return forms[i].frm->handleLoadEvent();
        }
    }
    return false;
}

/*  the function follows the following steps:
	o  find the location of the given form
	o  find the location of the first form handling the same form_id
	o  if the two do not coincide, switch the form pointers so that the given form comes first */
void FormManager::bringToFront (UInt16 form_id, Form* form)
{
	int form_poz = -1, first_poz = -1;
	for(int i=0; i<max_forms;i++) {
		if(forms[i].form_id == form_id && first_poz == -1) first_poz = i;
		if(forms[i].frm == form) form_poz = i;
	}
	if(first_poz<form_poz){
		Form* tmp = forms[first_poz].frm;
		forms[first_poz].frm = forms[form_poz].frm;
		forms[form_poz].frm = tmp;
	}
}

Boolean FormManager::dispatchCloseEvent(UInt16 form_id)
{
    for(int i=0;i<max_forms;i++) {
        if(forms[i].form_id == form_id && forms[i].active) {
            return forms[i].frm->handleCloseEvent();
        }
    }
    return false;
}

Boolean FormManager::dispatchEvent(EventPtr event)
{
    //the event should be generated by an object in the form currently active
    UInt16 form_id = FrmGetActiveFormID();
    for(int i=0;i<max_forms;i++) {
        if(forms[i].form_id == form_id && forms[i].active) {
            return forms[i].frm->handleEvent(event);
        }
    }
    return false;
}
