/**	$Revision: 1.21 $
	Last updated: $Date: 2002/09/25 21:18:01 $

	Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "FreeCoins.h"
#include "CoinsPref.h"
#include "AccountsDB.h" 
#include "TransactionsDB.h" 
#include "SplitsDB.h"
#include "CurrencyDB.h"
#include "FormManager.h"
#include "AccountsForm.h"
#include "FreeCoinsRsc.h"
#include "GenericInputForm.h"

TransactionsDB trsdb;
AccountsDB accdb;
SplitsDB spldb;
CurrencyDB curdb;

Boolean ApplicationHandleEvent( EventPtr event )
{
	Boolean	eventHandled = true;
	UInt16 formID;
	FormPtr frm;
	
	switch (event->eType) {
		case frmLoadEvent:
			// Load the form resource.		
			formID = event->data.frmLoad.formID;                
			if(FormManager::instance()->dispatchLoadEvent(formID)) {
				return true;
			}
			break;

		case frmCloseEvent:
			//first try our form manager
			if(FormManager::instance()->dispatchCloseEvent(event->data.frmClose.formID)) {
				return true;
			}
			//if that failed carry on with the default close operation
			frm = FrmGetFormPtr(event->data.frmClose.formID);
			FrmEraseForm(frm);
			FrmDeleteForm(frm);
			eventHandled = true;
			break;

		default:
			eventHandled = false;
	}	
	return eventHandled;
}


void ApplicationEventLoop(void)
{
	UInt16 error;
	EventType event;

	do {
		EvtGetEvent(&event, evtWaitForever);
		if(!SysHandleEvent(&event)) //not system event
			if(!MenuHandleEvent(NULL,&event,&error))
                //our event handling function
				if(!ApplicationHandleEvent(&event))
                    if(!FormManager::instance()->dispatchEvent(&event)) 
                        FrmDispatchEvent(&event);
	} while (event.eType != appStopEvent);
}

void StartApplication( void )
{
	UInt32 datePref;

	//init the database pointers
	transactions_db = &trsdb;
	splits_db = &spldb;
	currency_db = &curdb;
	accounts_db = &accdb;

	//set some localisation parameters
	LocGetNumberSeparators((NumberFormatType) PrefGetPreference(prefNumberFormat),
						   &MiscFunc::cThousandSep, &MiscFunc::cDecimalSep);	
	datePref = PrefGetPreference(prefDateFormat);
	switch(datePref) {
		case dfDMYWithSlashes: 
			MiscFunc::bDayFirst = true; MiscFunc::cDateSep='/';
			break;
		case dfDMYWithDots: 
			MiscFunc::bDayFirst = true; MiscFunc::cDateSep='.';
			break;
		case dfDMYWithDashes: 
			MiscFunc::bDayFirst = true; MiscFunc::cDateSep='-';
			break;
		default:
		case dfYMDWithSlashes:
		case dfMDYWithSlashes: 
			MiscFunc::bDayFirst = false; MiscFunc::cDateSep='/';
			break;
		case dfYMDWithDashes: 
			MiscFunc::bDayFirst = false; MiscFunc::cDateSep='-';
			break;              
		case dfYMDWithDots: 
			MiscFunc::bDayFirst = false; MiscFunc::cDateSep='.';
			break;              
	}

	//update scheduled transactions
	transactions_db->schedule();
	accounts_db->updateVisibility();
}


void StopApplication( void )
{
	// Close all open forms,  this will force any unsaved data to
	// be written to the database. Also need to save the preferences and dispose of allocated memory
	FrmCloseAllForms();
	preferences.save();
	FormManager::instance()->suicide();
}

/*	Application entry point  */
UInt32	PilotMain( UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags )
{
	// If normal launch
	if ( cmd == sysAppLaunchCmdNormalLaunch )
		{
			// check password if desired
			if (preferences.get()->check_pass) {
				// just in case the pass was deactivated after the user set the
				// option in the prefs dialog
				MemHandle str_handle;
				if (!PwdExists()) {
					str_handle = DmGetResource (strRsc,PASS_ERR_STR);
					FrmCustomAlert(GENERIC_INFO,reinterpret_cast<char*> (MemHandleLock (str_handle)),NULL,NULL);
				} else {
					MemHandle msg_handle;
					while(1) {
						Char pwd_str[pwdLength];
						str_handle = DmGetResource (strRsc,PASS_TITLE_STR);
						msg_handle = DmGetResource (strRsc,PASS_MSG_STR);
						GenericInputForm g(reinterpret_cast<char*> (MemHandleLock (str_handle)),
										   reinterpret_cast<char*> (MemHandleLock (msg_handle)),pwd_str,pwdLength);
						if(g.doDialog()) {
							if(PwdVerify(pwd_str)) break;
						} else return 0;					
		        	}
					MemHandleUnlock (msg_handle);
					DmReleaseResource (msg_handle);
				}
				MemHandleUnlock (str_handle);
				DmReleaseResource (str_handle);
			}
			//we can start the application
			StartApplication();
			
			//go straight to the accounts form on startup
			AccountsForm accounts_form;
			accounts_form.popup();
	
			// run the event loop. returns to exit.		
			ApplicationEventLoop();
			StopApplication();
		}
	return 0;
}	

