/**	$Revision: 1.3 $
	Last updated: $Date: 2003/01/28 07:52:05 $

	Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
		
	$Log: GenericDB.cpp,v $
	Revision 1.3  2003/01/28 07:52:05  dizzy
	Attempt to fix transaction note problem
	
	Revision 1.2  2002/09/09 18:24:45  dizzy
	Some DB code cleanup. renamed acname to name and acc_type to type in the accountStructure
	
	Revision 1.1  2002/06/14 10:16:24  dizzy
	Migration to autotools and internationalisation support
	
	Revision 1.1  2001/07/28 19:27:16  chaese
	Eliminated code duplication by moving database functions that don't depend
	on record structure into a generic base class. Saves 4.5K in program size! */

#include "GenericDB.h"
#include "FreeCoinsRsc.h"

UInt32 GenericDB::getRecordUid(UInt16 id)
{
	UInt32 acc_uid;
	
	DmRecordInfo(ref,id,NULL,&acc_uid,NULL);
	return acc_uid;
}

UInt16 GenericDB::getRecordId(UInt32 uid)
{	
	UInt16	id;
	
	DmFindRecordByID(ref,uid,&id);
	return id;
}

UInt16 GenericDB::getSelectedRecord() 
{ 
	if(!isRecordSelected()) {
		MemHandle str_handle = DmGetResource (strRsc,NO_SEL_REC_STR);
		ErrFatalDisplay(reinterpret_cast<char*> (MemHandleLock (str_handle)));
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
	}
	return selected_record; 
}	

UInt32 GenericDB::getSelectedUid()
{ 
	if(!isRecordSelected()) {
		MemHandle str_handle = DmGetResource (strRsc,NO_SEL_REC_STR);
		ErrFatalDisplay(reinterpret_cast<char*> (MemHandleLock (str_handle)));
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
	}
	return getRecordUid(selected_record); 
}

UInt16 GenericDB::getRecordCategory(UInt16 id)
{
	UInt16	attr;

	DmRecordInfo(ref,id,&attr,NULL,NULL);
	attr &= dmRecAttrCategoryMask;	
	return attr;
}

UInt16 GenericDB::getRecordCategory(UInt32 uid)
{
	return getRecordCategory(getRecordId(uid));
}

void GenericDB::setRecordCategory(UInt16 id, UInt16 cat)
{
	UInt16	attr;

	DmRecordInfo(ref,id,&attr,NULL,NULL);
	attr &= ~dmRecAttrCategoryMask;	
	attr |= cat & dmRecAttrCategoryMask;
	DmSetRecordInfo(ref,id,&attr,NULL);			
}	

void GenericDB::setRecordCategory(UInt32 uid, UInt16 cat)
{
	setRecordCategory(getRecordId(uid),cat);
}

UInt16 GenericDB::findCategory(Char* cat_name)
{
	return CategoryFind(ref,cat_name);
}

void GenericDB::getCategoryName(UInt16 cat_id, Char* cat_name)
{
	CategoryGetName(ref, cat_id, cat_name);
}

GenericDB::GenericDB(DmOpenRef db_open_ref)
{
	current_cat=dmAllCategories;
	ref=db_open_ref;
	unselectRecord();
}

GenericDB::GenericDB(UInt32 type, UInt32 creator, Char* db_name)
{
	UInt32	mode = dmModeReadWrite;
	MemPtr	c;

	created=false;
	current_cat=dmAllCategories;
	
	// Find the database.  If it doesn't exist, create it.
	openOrCreate(type,creator, mode,0, db_name);

	//have to initialise the application info
	c = getLockedAppInfo();
	MemPtrUnlock(c);
	unselectRecord();
}

GenericDB::GenericDB(UInt32 type, UInt32 creator, Char* db_name,
	UInt16 categ_string)
{
	UInt32	mode = dmModeReadWrite;
	MemPtr	c;

	current_cat=dmAllCategories;
	created=false;
	
	// Find the database.  If it doesn't exist, create it.
	openOrCreate(type,creator, mode,0, db_name);

	//have to initialise the application info
	c = getLockedAppInfo(categ_string);
	MemPtrUnlock(c);	
	unselectRecord();
}

GenericDB::~GenericDB()
{
	DmCloseDatabase(ref);
}

bool GenericDB::deleteRecord(UInt16	id)
{
	bool res = (DmRemoveRecord(ref,id) == errNone);
	//if the current record was removed, unselect the record
	if(selected_record == id) unselectRecord();
	return res;
}

bool GenericDB::deleteRecord(UInt32	uid)
{
	return deleteRecord(getRecordId(uid));
}

Err GenericDB::openOrCreate(
	UInt32 type, UInt32 creator, UInt16 mode, UInt16 cardNo, Char* name)
{
	Err	err=0;
	UInt16 dbAttr;
	LocalID dbId;

	created = false;
	ErrFatalDisplayIf(cardNo,"cardno not 0");
	if (! DmFindDatabase(cardNo,name) ) { //database does not exist
		err = DmCreateDatabase(cardNo, name, creator, type, false);
		if(err) {
			MemHandle str_handle = DmGetResource (strRsc,DB_CREATE_FAIL_STR);
			ErrFatalDisplay(reinterpret_cast<char*> (MemHandleLock (str_handle)));

			MemHandleUnlock (str_handle);
			DmReleaseResource (str_handle);
			return err;
		}
		created = true; //set flag
	}

	//open database (has to exist at this point
	ref = DmOpenDatabase(cardNo,DmFindDatabase(cardNo,name),mode);

	if (! ref) return DmGetLastErr();

	// tell HotSync to make backups
	DmOpenDatabaseInfo(ref, &dbId, NULL, NULL, &cardNo, NULL);
	DmDatabaseInfo(cardNo, dbId, NULL, &dbAttr, NULL, NULL, NULL, NULL,
		NULL, NULL, NULL, NULL, NULL);
	dbAttr |= dmHdrAttrBackup;
	DmSetDatabaseInfo(cardNo, dbId, NULL, &dbAttr, NULL, NULL, NULL, NULL,
		NULL, NULL, NULL, NULL, NULL);

	return err;
}

MemPtr GenericDB::appInfoInit(LocalID dbID,UInt16 cardNo)
{
	UInt16			wordValue;
	LocalID			appInfoID;
	appInfoType*	nilP = 0;
	appInfoType* 	appInfoP;

	MemHandle h;

	
	h = DmNewHandle(ref, sizeof (appInfoType));
	ErrFatalDisplayIf (!h,  "Cannot allocate app info block");

	//get the local id of the newly allocated appinfo record
	appInfoID = MemHandleToLocalID (h);
	//record the id
	DmSetDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, NULL, NULL, NULL,NULL, &appInfoID, NULL, NULL, NULL);
	
	appInfoP = (appInfoType*) MemLocalIDToLockedPtr(appInfoID, cardNo);

	// Clear the app info block.
	DmSet (appInfoP, 0, sizeof(appInfoType), 0);

	// I don't know what this field is used for.
	wordValue = 0xFFFF;
	DmWrite (appInfoP, (UInt32)&nilP->dirtyAppInfo, &wordValue,
		sizeof(appInfoP->dirtyAppInfo));

	return appInfoP;
}

MemPtr GenericDB::appInfoInit(LocalID dbID,UInt16 cardNo, UInt16 categ_string)
{
	UInt16			wordValue;
	LocalID			appInfoID;
	appInfoType*	nilP = 0;
	appInfoType* 	appInfoP;

	MemHandle h;

	
	h = DmNewHandle(ref, sizeof (appInfoType));
	ErrFatalDisplayIf (!h,  "Cannot allocate app info block");

	//get the local id of the newly allocated appinfo record
	appInfoID = MemHandleToLocalID (h);
	//record the id
	DmSetDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, NULL, NULL, NULL,NULL, &appInfoID, NULL, NULL, NULL);
	
	appInfoP = (appInfoType*) MemLocalIDToLockedPtr(appInfoID, cardNo);

	// Clear the app info block.
	DmSet (appInfoP, 0, sizeof(appInfoType), 0);

	// Initialize the categories.
	CategoryInitialize ((AppInfoPtr) appInfoP, categ_string);

	// I don't know what this field is used for.
	wordValue = 0xFFFF;
	DmWrite (appInfoP, (UInt32)&nilP->dirtyAppInfo, &wordValue,
		sizeof(appInfoP->dirtyAppInfo));

	// Initialize the sort order.
//	DmSet (appInfoP, (UInt32)&nilP->sortOrder, sizeof(appInfoP->sortOrder),		 soName);

	return appInfoP;
 }

MemPtr GenericDB::getLockedAppInfo()
{
	UInt16	cardNo;
	LocalID	dbID;
	LocalID	appInfoID;
	Err	err;

	err = DmOpenDatabaseInfo(ref, &dbID, NULL, NULL,&cardNo, NULL);
	ErrFatalDisplayIf (err,  "Error while getting application info block");
	err = DmDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, NULL, NULL,NULL, NULL, &appInfoID, NULL, NULL, NULL);
	ErrFatalDisplayIf (err,  "Error while getting application info block");
	
	if(appInfoID==NULL) return appInfoInit(dbID,cardNo);
	//a bit useless - to be removed unless code will be inserted here
	return MemLocalIDToLockedPtr(appInfoID, cardNo);
}

void GenericDB::selectRecord(UInt32 uid) 
{ 
	selectRecord(getRecordId(uid)); 
}

void GenericDB::selectRecord(UInt16 id)
{ 
	selected_record=id;
	record_selected=true; 
}

MemPtr GenericDB::getLockedAppInfo(UInt16 categ_string)
{
	UInt16	cardNo;
	LocalID	dbID;
	LocalID	appInfoID;
	Err	err;

	err = DmOpenDatabaseInfo(ref, &dbID, NULL, NULL,&cardNo, NULL);
	ErrFatalDisplayIf (err,  "Error while getting application info block");
	err = DmDatabaseInfo(cardNo, dbID, NULL, NULL, NULL, NULL, NULL,NULL, NULL, &appInfoID, NULL, NULL, NULL);
	ErrFatalDisplayIf (err,  "Error while getting application info block");
	
	if(appInfoID==NULL) return appInfoInit(dbID, cardNo, categ_string);
	//a bit useless - to be removed unless code will be inserted here
	return MemLocalIDToLockedPtr(appInfoID, cardNo);
}

DmOpenRef GenericDB::getRef()
{
	return ref;
}

Boolean GenericDB::isNew()
{
	return created;
}

void GenericDB::unselectRecord()
{
	record_selected = false;
}

Boolean GenericDB::isRecordSelected()
{
	return record_selected;
}

void GenericDB::writeAndAdvance(MemPtr s, UInt32& offset, Char* str)
{
    
	DmStrCopy(s, offset, str);
	offset += StrLen(str)+1;
}







