/**	$Revision: 1.9 $
	Last updated: $Date: 2002/09/11 12:46:45 $
	Copyright (C) 2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "MiscFunc.h"
namespace MiscFunc
{
	char cThousandSep;
	char cDecimalSep;
	Boolean bDayFirst;
	char cDateSep;
};

#include "CoinsPref.h"
#include "FreeCoinsRsc.h"

Boolean MiscFunc::osVerGe(UInt32 version_no)
{
	UInt32 dev_ver;
	FtrGet(sysFtrCreator,sysFtrNumROMVersion, &dev_ver);
	if(dev_ver<version_no) return false;
	return true;
}

Char*	MiscFunc::Pence2String(Int32 pence)
{
	Char* s = (Char*) MemPtrNew(30), *t;
	Int32 pennies, ones = 0, thousands = 0, millions = 0;

	t = s;
	if (pence<0) {
		pence *= -1;
		*t++ = '-';
	}
	pennies = pence % 100; pence /= 100;
	if (preferences.get()->use_thousands_sep) {
		// break amount into groups of thousands
		ones = pence % 1000; pence /= 1000;
		thousands = pence % 1000;
		millions = pence/1000;
	}
	else {
		// lump everything into one group
		ones = pence;
	}
	if (!thousands && !millions)
		StrPrintF(t, "%ld.%02ld",ones,pennies);
	else if (!millions)
		StrPrintF(t, "%ld,%03ld.%02ld",thousands,ones,pennies);
	else
		StrPrintF(t, "%ld,%03ld,%03ld.%02ld",millions,thousands,ones,pennies);

	StrLocalizeNumber(s, cThousandSep, cDecimalSep);
	return s;
}

void MiscFunc::DrawCharsRightAlign(const Char* s, const RectangleType* bounds)
{
	Int16 offset;
	RectangleType oldcliprect, newcliprect;

	WinGetClip(&oldcliprect);
	RctGetIntersection(bounds, &oldcliprect, &newcliprect);
	WinSetClip(&newcliprect);
	offset = bounds->extent.x - FntCharsWidth(s, StrLen(s)) - 1;
	if (offset<0) offset = 0;
	WinDrawChars(s,StrLen(s),bounds->topLeft.x+offset, bounds->topLeft.y);
	WinSetClip(&oldcliprect);
}
 
Int32	MiscFunc::String2Pence(Char* sum)
{
	Int32 result = 0, scale = 0, sign = 1;
	Char c;

	// return if null pointer is given	
	if (!sum) return 0;

	/* The following code is an almost straightforward parser to scan the
	string and build a number from it. Have fun figuring out how it works! :)
    */
	if (*sum=='-') {
		sign = -1; sum++;
	}
	while ((c = *sum++)) {
		if (c==cThousandSep) continue;
		if (c==cDecimalSep && !scale) { scale = 100; continue; }
		if (c<'0'||c>'9') break;
		if (scale==1) break;
		scale /= 10;
		result = result * 10 + (c - '0');
	}
	if (!scale) scale = 100;
	return (sign * scale * result);
}

void*	MiscFunc::GetObjectPtr(UInt16 objID)
{
	FormPtr frm;
	frm=FrmGetActiveForm();
	return (FrmGetObjectPtr (frm, FrmGetObjectIndex (frm,objID)));
}

void*	MiscFunc::GetObjectPtrByForm(FormPtr frm,UInt16 objID)
{
	return (FrmGetObjectPtr (frm, FrmGetObjectIndex (frm,objID)));
}

Char*	MiscFunc::GetTextPtr(UInt16 objID)
{
	FieldPtr fp=(FieldPtr) GetObjectPtr(objID);
	
	return reinterpret_cast <char*>(FldGetTextPtr(fp));
}

FieldPtr MiscFunc::SetFieldTextFromHandle(Int16 fieldID, MemHandle txtH)
{
   MemHandle	oldTxtH;
   FormPtr	frm = FrmGetActiveForm();
   FieldPtr	fldP;


   // get the field and the field's current text handle.
   fldP = (FieldPtr) GetObjectPtrByForm(frm, fieldID);
   ErrNonFatalDisplayIf(!fldP, "Cannot set text field: missing field");
   oldTxtH = FldGetTextHandle(fldP);
   
   // set the field's text to the new text.
   FldSetTextHandle(fldP, txtH);
   FldDrawField(fldP);

   // free the handle AFTER we call FldSetTextHandle().
   if (oldTxtH) 
      MemHandleFree(oldTxtH);
   
   return fldP;
} 

FieldPtr MiscFunc::SetFieldTextFromStr(Int16 fieldID, const Char* str)
{
   MemHandle      txtH;
   
   // get some space in which to stash the string.
   txtH  = MemHandleNew(StrLen(str) + 1);
   if (!txtH)
      return NULL;

   // copy the string to the locked handle.
   StrCopy((Char*) MemHandleLock(txtH), str);

   // unlock the string handle.
   MemHandleUnlock(txtH);
   
   // set the field to the handle
   return SetFieldTextFromHandle(fieldID, txtH);
} 

FieldPtr MiscFunc::SetFieldTextFromInt(Int16 fieldID, const Int32 i)
{
   MemHandle      txtH;
   
   // get some space in which to stash the string.
   txtH  = MemHandleNew(16);
   if (!txtH) 
      return NULL;

   // copy the string to the locked handle.
   StrIToA((Char*) MemHandleLock(txtH),i);

   // unlock the string handle.
   MemHandleUnlock(txtH);
   
   // set the field to the handle
   return SetFieldTextFromHandle(fieldID, txtH); 
} 

//clears the text from a field:
void  MiscFunc::ClearFieldText(Int16 fieldID)
{
   SetFieldTextFromHandle(fieldID, NULL);
}

void MiscFunc::updateDateField(UInt16 date_field_id, DateType new_date, Char* date_txt)
{
	DateFormatType	date_format;
	//stores the date text
		
	date_format = (DateFormatType) PrefGetPreference(prefDateFormat);
	DateToAscii(new_date.month, new_date.day, new_date.year+1904, date_format, date_txt);
	CtlSetLabel((ControlType*) GetObjectPtrByForm(FrmGetActiveForm(),date_field_id),date_txt);
}

 
void MiscFunc::selectDate(DateType& dest, UInt16 date_field_id, Char* date_txt)
{
	Boolean date_changed;
	Int16 	year,month,day;
	
	year=dest.year+1904;
	month=dest.month;
	day=dest.day;
	
	MemHandle str_handle = DmGetResource (strRsc,CHOOSE_DATE_STR);
	date_changed=SelectDay(selectDayByDay,&month,&day,&year,reinterpret_cast<char*> (MemHandleLock (str_handle)));
	MemHandleUnlock (str_handle);
	DmReleaseResource (str_handle);
	if(date_changed) {
		dest.year=year-1904;
		dest.month=month;
		dest.day=day;
		updateDateField(date_field_id,dest,date_txt);
	}
}

Int16 MiscFunc::compareDates(DateType date1, DateType date2)
{
	return DateToDays(date1) - DateToDays(date2);
}										

Boolean MiscFunc::validNumberStr(const Char *str, Boolean allow_minus)
{
	// ignore trailing blanks
	while (*str==' ') str++;

	// ignore minus sign if it's allowed
	if (allow_minus && *str=='-') str++;

	while (*str) {
		if ((*str<'0' || *str>'9') && *str != cDecimalSep
				&& *str !=cThousandSep)
			return false;
		str++;
	}
	return true;
}

void MiscFunc::PopupListSetSelection(UInt16 listId, UInt16 trigId, Int16 sel_index)
{
    ListType *list;
    ControlType *trig;
	FormType *frm = FrmGetActiveForm();
    list = (ListType*)FrmGetObjectPtr(frm, FrmGetObjectIndex(frm,listId));
    trig = (ControlType*)FrmGetObjectPtr(frm, FrmGetObjectIndex(frm,trigId));
    LstSetSelection(list, sel_index);
    CtlSetLabel(trig, LstGetSelectionText(list, LstGetSelection(list)));
}
