/**	$Revision: 1.33 $
	Last updated: $Date: 2002/09/25 21:18:01 $

	Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef __MISC_FUNC__
#define __MISC_FUNC__

#include <PalmOS.h>

/// application creator
#define FREE_COINS_CREATOR	'DyCo'

///version string for os 3.5
#define palmos_3_5 0x03503000 
///version string for os 3.1
#define palmos_3_1 0x03103000 

/// section definition for form code
#define FORM_SECTION1 __attribute__ (( section ("forms1")))

/// section definition for form code
#define FORM_SECTION2 __attribute__ (( section ("forms2")))

/// section definition for database code
#define DB_SECTION1 __attribute__ (( section ("dbs1")))

/// section definition for database code
#define DB_SECTION2 __attribute__ (( section ("dbs2")))

/// contains various functions used throughout the program
namespace MiscFunc {
	///Number format: Thousands separator (currently unused)
	extern char cThousandSep;
	///Number format: Decimal separator
	extern char cDecimalSep;
	///date format, print day first?
	extern Boolean bDayFirst;
	///date format, character between day and month
	extern char cDateSep;

	/**
		checks if the PalmOS version on the current device is greater or equal to the one specified.
		for example if(osVerGe(palmos_3_5)) DoSomething();
	
		@param version_no the version number - pass it one of the variables defined above (palmos_*****)
		@return true if the os has a verno greater or equal to the one specified, false otherwise
	*/
	Boolean osVerGe(UInt32 version_no);
	
	/**
		Get a 32bit int containg a sum of money in pence and return a pointer
		to a proper formatted string.

		the pointer has to be deallocated after use with MemPtrFree

	*/
	Char* Pence2String(Int32 pence) FORM_SECTION2;

	/**
		draw right aligned text
		@param s string to be drawn
		@param bounds rectangle inside which the text will be displayed
	*/
	void DrawCharsRightAlign(const Char* s, const RectangleType* bounds) FORM_SECTION2;

	/**
		Get a string containing a sum of money and convert it to pence
		@param sum the sting containing the formatted amount
		@return	the amount of pence in the sum
	*/
	Int32 String2Pence(Char* sum) FORM_SECTION2;

	/**
		returns a pointer to a given object in the current form
	*/
	void* GetObjectPtr(UInt16 objID) FORM_SECTION2;

	/**
		returns a pointer to a given object in a given form
	*/
	void*	GetObjectPtrByForm(FormPtr frm,UInt16 objID) FORM_SECTION2;

	/**
		returns pointer to the text in a given field in the current form
	*/
	Char*	GetTextPtr(UInt16 objID) FORM_SECTION2;

	/**  fills in a text field from a memory handle
		 @param fieldID id of the field to be filled
		 @param txtH memory handle to the text to be placed in the field
		 @return pointer to the field */
	FieldPtr SetFieldTextFromHandle(Int16 fieldID, MemHandle txtH);

	/// Allocates new handle and copies incoming string
	FieldPtr SetFieldTextFromStr(Int16 fieldID, const Char* str) FORM_SECTION2;

	/// Allocates new handle and copies incoming integer
	FieldPtr SetFieldTextFromInt(Int16 fieldID, const Int32 i) FORM_SECTION2;

	///clears the text from a field:
	void  ClearFieldText(Int16 fieldID) FORM_SECTION2;

	/**
		fills in a date field (pushbutton or selectortrigger) with a date
		@param date_field_id ID of the field
		@param new_date date to be displayed
		@param date_txt pointer to an allocated buffer where the date text can be stored
	*/
	void updateDateField(UInt16 date_field_id, DateType new_date, Char* date_txt) FORM_SECTION2;

	/**
		call the system form to choose a new date
		@param dest the choosen date will be stored in here
		@param date_field_id the date choosen will be displayed in this field
		@param date_txt pointer to be passed to updateDateField
	*/
	void selectDate(DateType& dest, UInt16 date_field_id, Char* date_txt) FORM_SECTION2;
	
	/**
		compares two dates
		@param date1 1st date
		@param date2 2nd date
		@return -1 if date1<date2, 0 if date1==date2 and 1 otherwise
	*/
	Int16 compareDates(DateType date1, DateType date2) DB_SECTION2;

	/**
		check if a string contains a valid decimal number
		@param str the string
		@param minus_allowed boolean that indicates if the number may be negative
		@return true if the string is valid, false otherwise
	*/
	Boolean validNumberStr(const Char *str, Boolean allow_minus) DB_SECTION2;

	/**
		set list selection and update associated popup trigger label
		@param listId the ID of the list to set
		@param trigId the ID of the associated popup trigger
		@param sel_index index of the item to select
	*/
	void PopupListSetSelection(UInt16 listId, UInt16 trigId, Int16 sel_index) FORM_SECTION2;

};

///export by default all the functions/vars in this namespace
using namespace MiscFunc;

#endif
