/** $Revision: 1.3 $
    Last updated: $Date: 2003/01/28 07:52:05 $

    Copyright (C) 2002 Vlad Mereuta <dizzy@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "NoteForm.h"
#include "FormManager.h"
#include "FreeCoinsRsc.h"

NoteForm* NoteForm::_instance = NULL;
NoteForm note_form;

NoteForm* NoteForm::instance()
{
	if(_instance == NULL) {
		_instance = &note_form;
	}
	return _instance;
}

NoteForm::NoteForm() : Form (NOTE_FORM)
{ 
	_note = NULL;
}

NoteForm::~NoteForm()
{
	deleteNote();
	FormManager::instance()->deregisterForm(form_id);
}

bool NoteForm::updatedData(UInt16 form_id)
{
	return (form_id == _last_caller);
}

Char* NoteForm::getNote()
{
	//delete note if empty
	if (_note == NULL) return "";
	return _note;
}

/*  need to handle the case in which we are supplied pointer to our own note separatelly (i.e. do nothing whatsoever)
    in this case there is no copying/reallocating needed (even more, it is impossible ;-)) */
void NoteForm::initForm (Char* title_str, Char* note_str) 
{
	_title = title_str;
	if (note_str != NULL && note_str != _note) {
		deleteNote();
        _note = new Char[StrLen (note_str)+1];
		StrCopy (_note, note_str);
	}
	_last_caller = 0;
}

Boolean NoteForm::handleKeyDownEvent(WChar key_id, UInt16 modifiers) 
{
	switch(key_id) {
		case vchrPageUp:
			pageScroll (winUp);
			break;
		case vchrPageDown:
			pageScroll (winDown);
			break;
		default:
			return false;
	}
	return true;
}

Boolean NoteForm::handleFldChangedEvent()
{
	updateScrollBar();
	return true;
}

Boolean NoteForm::handleSclRepeatEvent(UInt16 new_value, UInt16 value)
{
	scroll (new_value - value, false);
	return false;
}

Boolean NoteForm::handleOpenEvent()
{
	//draw form
	FrmDrawForm(frm);
	if (_note != NULL) {
		SetFieldTextFromStr (NOTE_FIELD, _note);
	}
	if (_title != NULL) 
		if (StrLen (_title) > 0) {
			FrmCopyTitle(frm, _title);
		} else {
			MemHandle str_handle = DmGetResource (strRsc,NO_TITLE_NOTE_STR);
			FrmCopyTitle(frm,reinterpret_cast<char*> (MemHandleLock (str_handle)));
			MemHandleUnlock (str_handle);
			DmReleaseResource (str_handle);
		}
	FrmSetFocus (frm, getObjectIndex (NOTE_FIELD));
	FieldPtr fld = reinterpret_cast<FieldPtr>(GetObjectPtr (NOTE_FIELD));
	FieldAttrType attr;
	FldGetAttributes (fld, &attr);
	attr.hasScrollBar = true;
	FldSetAttributes (fld, &attr);

	return true;
}

void NoteForm::deleteNote()
{
	if (_note != NULL) {
 		delete _note;
		_note = NULL;
	}	
}

/*  when ok is pressed do the following:
	- copy field text into newly allocated field
	- set last_caller to caller_form_id (thus forms can check if they called this form previously)
	- update caller form (which can now get the text)
	- return to previous form
    when cancel is pressed:
	- set last_caller to 0; this way forms checking for updates will ignore the NoteForm
	when delete is pressed:
	- allocated text field set to NULL
	- last_caller to caller_form_id */
Boolean NoteForm::handleCtlSelectEvent (UInt16 control_id)
{
	Boolean handled = true;
	
	switch (control_id) {
		case NOTE_OK:
			_last_caller = caller_form_id;
			deleteNote();
			if (GetTextPtr(NOTE_FIELD) != NULL)
				if (StrLen (GetTextPtr(NOTE_FIELD)) != 0) {
                    _note = new Char[ StrLen (GetTextPtr(NOTE_FIELD)) + 1];
					StrCopy (_note, GetTextPtr (NOTE_FIELD));
				}
			FrmUpdateForm (caller_form_id, 5);
			FrmReturnToForm (0);
			break;
		case NOTE_CANCEL:
			_last_caller = 0;
			FrmReturnToForm (0);
			break;
		case NOTE_DELETE:			
			_last_caller = caller_form_id;
			deleteNote();			
			FrmUpdateForm (caller_form_id, 5);
			FrmReturnToForm (0);
			break;
	}
	return handled;
}

void NoteForm::updateScrollBar()
{
	UInt16 scrollPos;
	UInt16 textHeight;
	UInt16 fieldHeight;
	Int16 maxValue;
	FieldPtr fld;
	ScrollBarPtr bar;

	fld = reinterpret_cast<FieldPtr>(GetObjectPtr (NOTE_FIELD));
	bar = reinterpret_cast<ScrollBarPtr>(GetObjectPtr (NOTE_SCROLLBAR));
	
	FldGetScrollValues (fld, &scrollPos, &textHeight,  &fieldHeight);

	if (textHeight > fieldHeight) {
		// On occasion, such as after deleting a multi-line selection of text,
		// the display might be the last few lines of a field followed by some
		// blank lines.  To keep the current position in place and allow the user
		// to "gracefully" scroll out of the blank area, the number of blank lines
		// visible needs to be added to max value.  Otherwise the scroll position
		// may be greater than maxValue, get pinned to maxvalue in SclSetScrollBar
		// resulting in the scroll bar and the display being out of sync.
		maxValue = (textHeight - fieldHeight) + FldGetNumberOfBlankLines (fld);
	} else if (scrollPos)
		maxValue = scrollPos;
	else
		maxValue = 0;

	SclSetScrollBar (bar, scrollPos, 0, maxValue, fieldHeight-1);
}


void NoteForm::scroll (Int16 lines_to_scroll, Boolean update_scroll_bar)
{
	UInt16 blank_lines;
	FieldPtr fld;
	
	fld = reinterpret_cast<FieldPtr>(GetObjectPtr (NOTE_FIELD));
	blank_lines = FldGetNumberOfBlankLines (fld);

	if (lines_to_scroll < 0)
		FldScrollField (fld, -lines_to_scroll, winUp);
	else if (lines_to_scroll > 0)
		FldScrollField (fld, lines_to_scroll, winDown);
		
	// If there were blank lines visible at the end of the field
	// then we need to update the scroll bar.
	if (blank_lines && lines_to_scroll < 0 || update_scroll_bar) {
		updateScrollBar();
	}
}

void NoteForm::pageScroll (WinDirectionType direction)
{
	UInt16 lines_to_scroll;
	FieldPtr fld = reinterpret_cast<FieldPtr>(GetObjectPtr (NOTE_FIELD));
	
	if (FldScrollable (fld, direction)) {
		lines_to_scroll = FldGetVisibleLines (fld) - 1;
		if (direction == winUp)
			lines_to_scroll = -lines_to_scroll;
		scroll(lines_to_scroll, true);
	}
}
