/**	$Revision: 1.5 $
	Last updated: $Date: 2003/01/28 07:52:05 $

	Copyright (C) 2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef __NOTE_FORM__
#define __NOTE_FORM__

#include <PalmOS.h>

#include "Form.h"


/**	class contains code necessary to do note editing 
	The data edited by the note field is copied on OK/Delete events to a temporary location which survives after the
	form closes. This class is that it holds an internal buffer containing the edited note. The buffer is allocated
	dynamically and can be filled in before startup (so that a pre-written note is displayed). The contents of the
	buffer can be retrieved at any point, even after the form was closed. This is due to the fact that the actual form
	object for the entire life-time of the program.

	@note this form generates update code 5, whenever the calling form needs to check for an update in the note field
*/
class NoteForm : public Form 
{
public:
	/// constructor; does nothing
	NoteForm() FORM_SECTION2;
	/// destructor; removes the currently allocated string
	~NoteForm() FORM_SECTION2;
	
	///@return singleton reference to this object
	static NoteForm* instance() FORM_SECTION2;

	/** @return pointer to the stored note */
	Char* getNote() FORM_SECTION2;

	/** the starting string can be a pointer to a character strin or NULL, in which case
	    new note is assumed; currently stored string is disposed of and re-allocated
		The last_caller field is set to 0
		@param title_str pointer to the string to be used as title. Note that this string should be allocated somewhere
		else and kept allocated until the form closes, as we simply mirror the address and not the location
		@param note_str pointer to starting string, if any */
    void initForm (Char* title_str, Char* note_str) FORM_SECTION2;

	/** checks whether a given form id should update its data according to data stored by this form
		@param form_id id of the form which is checking for updates
	    @return true if data has been updated and the given form should fetch the new string */
	bool updatedData (UInt16 form_id) FORM_SECTION2;
	
private:
	///internal reference to this object
	static NoteForm* _instance;
	///contains the last valid form which called this form (i.e. to which we exited with ok/delete)
	UInt16 _last_caller;
	///current form title
	Char* _title;
	///data edited is stored here
    Char* _note;

    ///deletes the _note attribute, only if necessary
	void deleteNote() FORM_SECTION2;

	/** update the scroll bar with the values hinted by the note field */
	void updateScrollBar() FORM_SECTION2;
	
	/** scroll the note by a specified number of lines
		@param lines_to_scroll number of lines to scroll (positive for down, negative for up
		@param update_scroll_bar if true, force a scrollbar update */
	void scroll (Int16 lines_to_scroll, Boolean update_scroll_bar) FORM_SECTION2;
	
	/** scroll the note a whole page up or down
		@param direction direction to scrol in, can be winUp or winDown */
	void pageScroll (WinDirectionType direction) FORM_SECTION2;
	

	///called when key down event occurs
	Boolean handleKeyDownEvent(WChar key_id, UInt16 modifiers) FORM_SECTION2;
	///called when form open event occurs
	Boolean handleOpenEvent() FORM_SECTION2;
	/**	called when ctl select event occurs
            @param control_id id of the control for which the event was generated */
	Boolean handleCtlSelectEvent(UInt16 control_id) FORM_SECTION2;
	/** handles events generated when field are changed */
	Boolean handleFldChangedEvent() FORM_SECTION2;
	/** handles events generated by the scrollbar */
	Boolean handleSclRepeatEvent(UInt16 new_value, UInt16 value) FORM_SECTION2;
};

#endif
