/**	Code to access the database containing the Splits information

	$Revision: 1.8 $
	Last updated: $Date: 2002/09/20 12:21:31 $
	
	Copyright (C) 2001-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef __SPLITS_DB__
#define __SPLITS_DB__

#include "Database.h"

/// type of the Splits database
#define SPLITSDB_TYPE	'DATA'
/// db name
#define SPLITSDB_NAME	"FreeCoins-Splits"

///the packed structure of a split 
typedef struct {
	/// UID of the transaction that the split belongs to
	UInt32		trans_uid;
	/// UID of the affected account
	UInt32		acc_uid;
	/// signed amount by which the account is affected
	UInt32		amount;
	/// line number for sorting purposes
	UInt8		line_no;
} packedSplitStructure;

///unpacked structure of a Split
typedef struct {
	/// UID of the transaction that the split belongs to
    UInt32      trans_uid;
	/// UID of the affected account
    UInt32      acc_uid;
	/// signed amount by which the account is affected
    Int32      amount;
	/// line number for sorting purposes
    UInt8       line_no;
} splitStructure;

///class to manipulate the splits database
class SplitsDB : public Database <splitStructure, packedSplitStructure>
{
 public:
	/// unpacks a Split record
	void	unpackRecord (splitStructure* spl, packedSplitStructure* packed_spl) DB_SECTION2;
	/// packs a Split record
	void	packRecord (splitStructure* spl, MemHandle db_entry) DB_SECTION2;
	/// initialize the database
	SplitsDB() DB_SECTION2;
	/** locate the first split that belongs to the given transaction
		@param trans_uid UID of the transaction
		@return the index of the first split within the transaction	*/
	UInt16 locateFirstSplit(UInt32 trans_uid) DB_SECTION2;
 
    /** set a record to cleared, update the affected account's cleared balance
        @param id id of the record to be set  */
    void    clearRecord(UInt16 id) DB_SECTION2;

    /** set a record to uncleared and update the affected account's cleared
		balance
        @param id id of the record to be set */
    void    unclearRecord(UInt16 id) DB_SECTION2;

	/**	creates a new record and update the balance of the involved account.
		The transaction that the split belongs to is responsible for keeping
		the number of splits it has up to date.
		@param s pointer to the structure containing the split data
		@param cleared indicates if this split affects the cleared balance
		@return id of the newly created record	*/
	UInt16	newRecord(splitStructure* s, Boolean cleared) DB_SECTION2;
	
	/**	deletes a record, given its id. The balance of the affected account
		is updated.
		The transaction that the split belonged to is responsible for keeping
		the number of splits it has up to date.
		@param id the id of the record to be deleted
		@param keep_balance if true then the balances of the account involved in the split will *not* be changed. Use only when necessary, as this
		results in unbalanced accounts		*/
	bool deleteRecord(UInt16 id, Boolean keep_balance = false) DB_SECTION2;

	/**	Compares two packed split structures.
		For parameters see the PalmOS reference	*/
	static Int16	compareRecords(packedSplitStructure* rec1,packedSplitStructure* rec2,Int16 other,SortRecordInfoPtr sr1,SortRecordInfoPtr sr2,MemHandle app_info) DB_SECTION2;
};

///reference through which this database can be accessed
extern SplitsDB* splits_db;

#endif
