/**	$Revision: 1.15 $
	Last updated: $Date: 2002/12/17 22:52:00 $

	Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "SplitsForm.h"
#include "FreeCoinsRsc.h"
#include "EditSplitForm.h"
#include "FormManager.h"

SplitsForm* SplitsForm::_instance = NULL;
SplitsForm splits_form;

SplitsForm* SplitsForm::instance()
{
	if(_instance==NULL) {
		_instance = &splits_form;
	}
	return _instance;
}	

SplitsForm::SplitsForm() : Form (SPLITS_FORM)
{
	splits_data = NULL;
	total_debits = total_credits = splits_count = 0;
}

SplitsForm::~SplitsForm()
{
	deleteAllSplits();
	FormManager::instance()->deregisterForm(form_id);
}

Boolean SplitsForm::checkIndex(UInt8 index)
{
	if (index>=splits_count) {
		ErrFatalDisplay("end of splits array exceeded");
		return false; 
	} else return true;
}

UInt16 SplitsForm::getCount(void)
{
	return splits_count;
}

void SplitsForm::setCount(UInt8 count)
{
	UInt8 i;
	Int32 amount;

	if (count==0) {
		deleteAllSplits();
	} else {
		if (!splits_data)
			splits_data = MemHandleNew(2*count*sizeof(UInt32));
		else
			MemHandleResize(splits_data, 2*count*sizeof(UInt32));
	}
	if (count>splits_count) {
		UInt32 *data_ptr;
		data_ptr = (UInt32*)MemHandleLock(splits_data);
		MemSet(data_ptr + (2*splits_count),
			2*(count-splits_count)*sizeof(UInt32), 0);
		MemHandleUnlock(splits_data);
	}
	splits_count = count;
	total_debits = total_credits = 0;
	for (i = 0; i<count; i++) {
		amount = getAmount(i);
		if (amount > 0) total_debits += amount;
		else total_credits -= amount;
	}	
}

UInt32 SplitsForm::getAccount(UInt8 index)
{
	UInt32 *data_ptr, result;
	if (checkIndex(index)) {
		data_ptr = (UInt32*)MemHandleLock(splits_data);
		result = data_ptr[2*index];
		MemHandleUnlock(splits_data);
		return result;
	}
	return 0;
}

Int32 SplitsForm::getAmount(UInt8 index)
{
	Int32 *data_ptr, result;
	if (checkIndex(index)) {
		data_ptr = (Int32*)MemHandleLock(splits_data);
		result = data_ptr[2*index+1];
		MemHandleUnlock(splits_data);
		return result;
	}
	return 0;
}
	
void SplitsForm::setAccount(UInt8 index, UInt32 acc_uid)
{
	UInt32 *data_ptr;
	if (checkIndex(index)) {
		data_ptr = (UInt32*)MemHandleLock(splits_data);
		data_ptr[2*index] = acc_uid;
		MemHandleUnlock(splits_data);
	}
}
		
void SplitsForm::setAmount(UInt8 index, Int32 amount)
{
	Int32 *data_ptr, prev_amount;
	if (checkIndex(index)) {
		data_ptr = (Int32*)MemHandleLock(splits_data);
		prev_amount = data_ptr[2*index+1];
		if (prev_amount > 0) total_debits -= prev_amount;
		else total_credits += prev_amount;
		data_ptr[2*index+1] = amount;
		if (amount > 0) total_debits += amount;
		else total_credits -= amount;
		MemHandleUnlock(splits_data);
	}
}

void SplitsForm::deleteSplit(UInt8 index)
{
	UInt32 *data_ptr;
	if (checkIndex(index)) {
		data_ptr = (UInt32*)MemHandleLock(splits_data);
		MemMove(data_ptr+2*index, data_ptr+2*(index+1),
			2*sizeof(UInt32)*(splits_count-1-index));
		MemHandleUnlock(splits_data);
		setCount(splits_count-1);
	}
}

void SplitsForm::deleteAllSplits()
{
	if(splits_data != NULL) {
		MemHandleFree (splits_data);
		splits_data = NULL;
	}
	splits_count = 0;
}

Boolean SplitsForm::handleOpenEvent()
{
	splits_tbl.initTable();
	FrmUpdateForm(form_id, 0);
	return true;
}

Boolean SplitsForm::handleKeyDownEvent (WChar key_id, UInt16 modifiers)
{
	ControlPtr ptr;
	
	switch(key_id) {
		case pageUpChr:			// Scroll up key pressed
			//link the hard button functionality to the soft up/down controls
			ptr = (ControlPtr) GetObjectPtr(SPLIT_PGUP);
			CtlHitControl(ptr);
			break;								
		case pageDownChr:		// Scroll down key pressed						
			ptr = (ControlPtr) GetObjectPtr(SPLIT_PGDN);
			CtlHitControl(ptr);
			break;
		default:
			return false;
	};
	
	return true;
}

Boolean SplitsForm::handleCtlSelectEvent (UInt16 control_id)
{
	switch(control_id) {
		case SPLIT_DONE:
			FrmUpdateForm(caller_form_id, 2);
			FrmReturnToForm(0);
			break;
		case SPLIT_NEW:
			edit_split_form.split_index = 255;
			popup(&edit_split_form);
			break;
		case SPLIT_PGUP:
			if(splits_tbl.pageUp()) FrmUpdateForm(form_id,1);
			break;
		case SPLIT_PGDN:
			if(splits_tbl.pageDown()) FrmUpdateForm(form_id,1);
			break;
		default:
			return false;
	}
	return true;
}

Boolean SplitsForm::handleUpdateEvent (UInt16 update_code)
{
	splits_tbl.loadData();

	if(update_code == 0 || update_code == frmRedrawUpdateCode) {
		if (!osVerGe(palmos_3_5)) FrmEraseForm(frm);
		FrmDrawForm(frm);
	}

	if(update_code & 1) {
		TblMarkTableInvalid((TablePtr)GetObjectPtr(SPLITS_TABLE));
		TblRedrawTable((TablePtr)GetObjectPtr(SPLITS_TABLE));
	}

	// draw totals
	RectangleType rect;
	Char *s;
	RctSetRectangle(&rect, 65, 123, 45, 11);
	s = Pence2String(total_debits);
	DrawCharsRightAlign(s, &rect);
	MemPtrFree(s);
	RctSetRectangle(&rect, 110, 123, 45, 11);
	s = Pence2String(total_credits);
	DrawCharsRightAlign(s, &rect);
	MemPtrFree(s);
	WinDrawLine(0, 123, 156, 123);
	WinDrawLine(0, 133, 156, 133);

	if(update_code == 2) return true;
	splits_tbl.showScrollArrows();

	return true;
}


Boolean SplitsForm::handleTblSelectEvent(tblSelect& tbl_select)
{
	UInt8 index;
	index = TblGetRowID(tbl_select.pTable,tbl_select.row);
	TblUnhighlightSelection(tbl_select.pTable);
	edit_split_form.split_index = index;
	popup(&edit_split_form);
	return true;
}
	
