/**	$Revision: 1.8 $
	Last updated: $Date: 2002/09/20 12:21:31 $

	Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef __SPLITS_FORM__
#define __SPLITS_FORM__

#include <PalmOS.h>
#include "Form.h"
#include "SplitsTable.h"
#include "EditSplitForm.h"

/**  Class used to handle Split managment */
class SplitsForm : public Form
{	
public:
	///constructor - initializes variables
	SplitsForm() FORM_SECTION2;
	///destructor - disposes of allocated memory
	~SplitsForm() FORM_SECTION2;

	/** returns reference to SplitsForm instance. Only allowed way to use the class */
	static SplitsForm* instance() FORM_SECTION2;
	
	/// @return value of splits_count
	UInt16 getCount() FORM_SECTION2;

	/** sets the value of splits_count; also updates the total_debits and credits variables */
	void setCount(UInt8 count) FORM_SECTION2;

	/** returns the account associated with a given split
		@param index split index to get the account for
		@return uid of the account for given split */
	UInt32 getAccount(UInt8 index) FORM_SECTION2;

	/** sets the account associated with a given split index
		@param index split index to modify the account for
		@param acc_uid new account uid */
	void setAccount(UInt8 index, UInt32 acc_uid) FORM_SECTION2;

	/**  returns the amount associated with a given split index
		 @param index split index to get the amount for
		 @return amount for given split index */
	Int32 getAmount(UInt8 index) FORM_SECTION2;

	/**  sets the amount associated with a given split index
		 @param index index of the split to modify the amount for
		 @param amount new amount */
	void setAmount(UInt8 index, Int32 amount) FORM_SECTION2;

	/**  deletes a given split
		 @param index of the split to delete */
	void deleteSplit(UInt8 index) FORM_SECTION2;

	/**  deletes all splits 
	     @warning for some reason this method breaks horribly if placed in FORM_SECTION2!*/
	void deleteAllSplits() FORM_SECTION2;

	///handles update events (public in order to be able to force updates)
	Boolean handleUpdateEvent (UInt16 update_code) FORM_SECTION2;
	
private:
	/// table used to display and select splits
	SplitsTable splits_tbl;
	///total number of splits for current transaction
	UInt8 splits_count;
	/// handle to memory area containing the array of splits
	MemHandle splits_data;
	///total for debit splits
	UInt32 total_debits;
	///total for credit splits
	UInt32 total_credits;
	///object handling particular split editing
	EditSplitForm edit_split_form;
	///singleton reference to this form
	static SplitsForm* _instance;

	///draws form and table
	Boolean handleOpenEvent() FORM_SECTION2;
	///handles pgup/down keys
	Boolean handleKeyDownEvent (WChar key_id, UInt16 modifiers) FORM_SECTION2;
	///handles form controls
	Boolean handleCtlSelectEvent (UInt16 control_id) FORM_SECTION2;
	/**  checks whether the given split index is valid
		 @param index split index to check
		 @return true if valid, false otherwise */
	Boolean checkIndex(UInt8 index) FORM_SECTION2;
	///handles table events
	Boolean handleTblSelectEvent(tblSelect& tbl_select) FORM_SECTION2;
};

#endif
