/**	$Revision: 1.34 $
	Last updated: $Date: 2003/01/29 23:45:08 $
	
	Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "TransactionsDB.h"

#include <DateTime.h>
#include <StdIOPalm.h>
#include "SplitsDB.h"
#include "AccountsDB.h"
#include "MiscFunc.h" //macro defs
#include "CoinsPref.h"
#include "FreeCoinsRsc.h"

TransactionsDB* transactions_db;

UInt16 TransactionsDB::getClearedCategory()
{
	UInt16 cat;
	MemHandle str_handle = DmGetResource (strRsc,CLEARED_STR);
	cat = findCategory(reinterpret_cast<char*> (MemHandleLock (str_handle)));
	MemHandleUnlock (str_handle);
	DmReleaseResource (str_handle);
	return cat;
}

UInt16 TransactionsDB::getClUnclCategory()
{
	UInt16 cat;
	MemHandle str_handle = DmGetResource (strRsc,CL_UNCL_STR);
	cat = findCategory(reinterpret_cast<char*> (MemHandleLock (str_handle)));
	MemHandleUnlock (str_handle);
	DmReleaseResource (str_handle);
	return cat;
}

UInt16 TransactionsDB::getScheduledCategory()
{
	UInt16 cat;
	MemHandle str_handle = DmGetResource (strRsc,SCHEDULED_STR);
	cat = findCategory(reinterpret_cast<char*> (MemHandleLock (str_handle)));
	MemHandleUnlock (str_handle);
	DmReleaseResource (str_handle);
	return cat;
}

UInt16 TransactionsDB::getUnclearedCategory()
{
	UInt16 cat;
	MemHandle str_handle = DmGetResource (strRsc,UNCLEARED_STR);
	cat = findCategory(reinterpret_cast<char*> (MemHandleLock (str_handle)));
	MemHandleUnlock (str_handle);
	DmReleaseResource (str_handle);
	return cat;
}

Boolean	TransactionsDB::isCleared(UInt16 id)
{
	if(getRecordCategory(id)==getClearedCategory()) return true;
	else if(isScheduled(id)) return willBeCleared(id);
	return false;	
}

Boolean	TransactionsDB::isScheduled(UInt16 id)
{
	return (getRecordCategory(id)==getScheduledCategory());
}

Boolean	TransactionsDB::willBeCleared(UInt16 id)
{
	transactionStructure ts;	
	getUnpackedRecord(id,&ts);

	if(isScheduled (id))
		if(ts.sched.to_be_cleared) return true;
	
	return false;
}

void	TransactionsDB::clearRecord(UInt16 id)
{
	MemHandle				tr_h;
	transactionStructure 	ts;	
	UInt32 trans_uid;
	UInt16 index;
	UInt16 count;
	
	tr_h=getUnpackedRecord(id,&ts);	

	if(isScheduled(id)) {
		ts.sched.to_be_cleared=true;
		packRecord(&ts,tr_h);
		return;
	}		

	trans_uid = getRecordUid(id);		
	setRecordCategory(id,getClearedCategory());
	count = ts.num_splits;
	for (index=splits_db->locateFirstSplit(trans_uid);count>0;index++,count--){
		splits_db->clearRecord(index);
	}
}

void	TransactionsDB::unclearRecord(UInt16 id)
{
	MemHandle 				tr_h;
	transactionStructure	ts;	
    UInt32 trans_uid;
    UInt16 index;
    UInt16 count;

	tr_h=getUnpackedRecord(id,&ts);	
	
	if(getRecordCategory(id)==getScheduledCategory()) {
		ts.sched.to_be_cleared=false;
		packRecord(&ts,tr_h);
		return;
	}		
	
	setRecordCategory(id,getUnclearedCategory());
    trans_uid = getRecordUid(id);
    count = ts.num_splits;
    for (index=splits_db->locateFirstSplit(trans_uid);count>0;index++,count--){
        splits_db->unclearRecord(index);
    }
}


Boolean TransactionsDB::isBalanced(UInt32 trans_uid)
{
    transactionStructure ts;
    splitStructure s;
    UInt16 index;
    UInt16 count;
	Int32 total_effect=0;

    getUnpackedRecord(trans_uid,&ts);

    count = ts.num_splits;
	index=splits_db->locateFirstSplit(trans_uid);
	if (count == 2) {
		splits_db->getUnpackedRecord(index, &s);
		total_effect += (Int32) (s.amount * ts.exchange_rate);
		splits_db->getUnpackedRecord((UInt16)(index+1), &s);
		total_effect += s.amount;
	} else {
		for (;count>0;index++,count--){
		splits_db->getUnpackedRecord(index, &s);
		total_effect += s.amount;
		}
    }                                                   
	return (total_effect==0);
}

Int32 TransactionsDB::getAmount(UInt32 trans_uid, UInt32 acc_uid)
{
    transactionStructure ts;
    splitStructure s;
    UInt16 index;
    UInt16 count;

    getUnpackedRecord(trans_uid,&ts);

    count = ts.num_splits;
    for (index=splits_db->locateFirstSplit(trans_uid);count>0;index++,count--){
		splits_db->getUnpackedRecord(index, &s);
		if (s.acc_uid == acc_uid)
			return s.amount;
    }                                                   
	return 0;
}


UInt32 TransactionsDB::getPeerAcc(UInt32 trans_uid, UInt32 acc_uid)
{
    MemHandle               tr_h;
    transactionStructure    ts;
    splitStructure          s;
    UInt16 index;
    UInt16 count;

    tr_h=getUnpackedRecord(trans_uid,&ts);

    count = ts.num_splits;
	ErrFatalDisplayIf(count!=2, "getPeerAcc called for non-transfer");

    for (index=splits_db->locateFirstSplit(trans_uid);count>0;index++,count--){
        splits_db->getUnpackedRecord(index, &s);
        if (s.acc_uid != acc_uid)
            return s.acc_uid;
    }
	return 0;
}


void	TransactionsDB::getDate(UInt16 record, DateType * pDate)
{
	MemHandle					rech;
	packedTransactionStructure* pts;

	rech=DmQueryRecord(ref,record);
	ErrFatalDisplayIf(!rech, "rech is 0 - aborting");

	pts=(packedTransactionStructure*) MemHandleLock(rech);
	ErrFatalDisplayIf(!pts, "pts is 0 - aborting");

	pDate->month = pts->date.month;	
	pDate->day = pts->date.day;	
	pDate->year = pts->date.year;	

	MemHandleUnlock(rech);
}


void TransactionsDB::unpackRecord (transactionStructure* trans, packedTransactionStructure* packed_trans)
{
	Char* s = (Char*)packed_trans + sizeof(packedTransactionStructure);
	packedScheduleStructure *sched;
	
	trans->date = packed_trans->date;
	trans->num_splits = packed_trans->num_splits;
	trans->method = packed_trans->method;
	trans->exchange_rate = packed_trans->exchange_rate;
	trans->is_scheduled = packed_trans->is_scheduled;

	MemSet(&(trans->sched), sizeof(scheduledStructure), 0);
	if (trans->is_scheduled) {
		sched = (packedScheduleStructure *)s;
		s += sizeof(packedScheduleStructure);
    	trans->sched.repeat_interval = sched->repeat_interval;
		trans->sched.repeat_on = sched->repeat_on;
		trans->sched.frequency = sched->frequency;
		trans->sched.end_by_date = sched->end_by_date;
		trans->sched.end_date = sched->end_date;	
		trans->sched.repeat_times = sched->repeat_times;
		trans->sched.to_be_cleared = sched->to_be_cleared;
		trans->sched.paused = sched->paused;
	}

	trans->payee = s;
	s += StrLen(s) + 1;
	trans->num = s;
	s += StrLen(s) + 1;
	trans->note = s;
}

void TransactionsDB::packRecord (transactionStructure* trans, MemHandle db_entry)
{
	UInt32	length = 0;
	MemPtr	s;
	UInt32	offset = 0;
  	packedTransactionStructure pt;
	packedScheduleStructure ps;
    
	// figure out necessary size
	length = sizeof(packedTransactionStructure)
		+ StrLen(trans->payee) + StrLen(trans->num) 
		+ StrLen(trans->note) + 3 ;  // 3 string terminators


	pt.num_splits = trans->num_splits;
	pt.date = trans->date;
	pt.method = trans->method;
	pt.exchange_rate = trans->exchange_rate;
	pt.is_scheduled = trans->is_scheduled;
	pt.reserved = 0;
	if (trans->is_scheduled) {
		length += sizeof(packedScheduleStructure);
		ps.repeat_on = trans->sched.repeat_on;
		ps.frequency = trans->sched.frequency;
		ps.to_be_cleared = trans->sched.to_be_cleared;
		ps.end_by_date = trans->sched.end_by_date;
		ps.end_date = trans->sched.end_date;
		ps.repeat_times = trans->sched.repeat_times;
		ps.repeat_interval = trans->sched.repeat_interval;
		ps.paused = trans->sched.paused;
	}


    //make a backup copy of the note. This is to prevent the note being mangled (or maybe just moved) by MemHandleResize below
    Char* tp = new Char[StrLen (trans->note)+1];
    StrCopy (tp, trans->note);    
    
	// resize the VoidHand
    if (MemHandleResize(db_entry, length) == 0) {        

		// copy the fields
        s = MemHandleLock(db_entry);

		writeAndAdvance(s, offset, &pt);
		if (trans->is_scheduled) {
			writeAndAdvance(s, offset, &ps);
		}
		writeAndAdvance(s, offset, trans->payee);		
		writeAndAdvance(s, offset, trans->num);
        //        writeAndAdvance(s, offset, trans->note);		
        writeAndAdvance (s, offset, tp);
		MemHandleUnlock(db_entry);
	} else {
		ErrFatalDisplay ("Resizing Transaction record failed");
	}
    delete tp;
}

TransactionsDB::TransactionsDB() : Database <transactionStructure, packedTransactionStructure> (TRANSACTIONSDB_TYPE, FREE_COINS_CREATOR,
		TRANSACTIONSDB_NAME, TRANSACTIONS_CATEGORIES)
{
	current_cat=preferences.get()->trans_view;
}

UInt16	TransactionsDB::newRecord(transactionStructure* s, Boolean cleared, Boolean scheduled)
{
	MemHandle					h,p;
	UInt16						index=0;

	if(scheduled) {
		s->sched.to_be_cleared = cleared;
	}
	
	s->is_scheduled = scheduled;

	h=DmNewHandle(ref,1);
	ErrFatalDisplayIf(!ref,"Ref is 0");
    
	packRecord(s,h);

	if(!h) {
		MemHandle str_handle = DmGetResource (strRsc,CLEARED_STR);
		ErrFatalDisplay(reinterpret_cast<char*> (MemHandleLock (str_handle)));
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
	}

	p= (MemHandle) MemHandleLock(h);
	index=DmFindSortPosition(ref,p, 0, (DmComparF *) compareRecords, 0);
	//have to use Ptr and not handle, probably because FindSortPosition locks the handle
	MemPtrUnlock(p);
	
	DmAttachRecord(ref,&index,h,0);	

	selectRecord(index);
	
	if(scheduled) {
		setRecordCategory(index,getScheduledCategory());		
		schedule();      
		return index;
	}
	//set the category
	if(cleared) setRecordCategory(index,getClearedCategory());
	else setRecordCategory(index,getUnclearedCategory());

	return index;
}

bool TransactionsDB::deleteRecord(UInt16 id, Boolean keep_balance)
{
	transactionStructure		t;
	Boolean 					cleared;
	UInt16 index, count;
	UInt32 trans_uid;

	if (isScheduled(id)) {
		//deleting a scheduled transaction shouldn't back out account balances
		keep_balance = true;
	}

	getUnpackedRecord(id,&t);

	cleared = (isCleared(id));
    trans_uid = getRecordUid(id);
    count = t.num_splits;
	//  for (index=splits_db->locateFirstSplit(trans_uid);count>0;index++,count--){
    for (index=splits_db->locateFirstSplit(trans_uid);count>0;count--) {
		//delete splits while maintaing the requested balance policy
        if(!splits_db->deleteRecord(index,keep_balance)) ErrFatalDisplay ("Fatal inconsistency while deleting split");
    }

	return Database<transactionStructure, packedTransactionStructure>::deleteRecord(id);	
}

Int16	TransactionsDB::compareRecords(packedTransactionStructure* rec1,packedTransactionStructure* rec2,Int16 other,SortRecordInfoPtr sr1,SortRecordInfoPtr sr2,MemHandle app_info)
{
	return compareDates(rec2->date,rec1->date);
}	

void TransactionsDB::purgeTransactions(DateType last_date)
{	
	//transaction are sorted either ascending or descending by date; we will just go sequentially through all the database and remove the transactions
	//which have a date which satisfies the given conditions. Since this operation is only done once every (few) months it should not be a big problem	
	
	UInt16 record_id;
	transactionStructure ts;

  	for(record_id=0;record_id<DmNumRecords(ref);record_id++) {
		getUnpackedRecord(record_id,&ts);

		if(compareDates(ts.date,last_date) < 0 && ! isScheduled(record_id)) {
			//if the date is ok and record not scheduled, delete it
			deleteRecord(record_id, true);
            record_id--;
		}
	}
}

Boolean TransactionsDB::calculateNextDate(UInt16 id)
{
	transactionStructure	t;
	MemHandle				rec_h;
	DateType				   new_date;
	
   if(!isScheduled(id)) ErrFatalDisplay("Transaction is not scheduled. This should not happen.");
   
   rec_h=transactions_db->getUnpackedRecord(id,&t); // get the transaction record; current (next) date is stored in t.date
	
	switch (t.sched.repeat_interval) {
		case rep_day: 
			DateDaysToDate(DateToDays(t.date)+t.sched.frequency,&new_date);
			break;
		case rep_week:
			DateDaysToDate(DateToDays(t.date)+t.sched.frequency*7,&new_date);
			break;
		case rep_month:
            //after looking at both ways to solve this I decided that this one looks better :-)
			new_date.year=t.date.year+(t.date.month+t.sched.frequency-1) / 12;
			new_date.month=(t.date.month+t.sched.frequency-1) % 12 + 1;
            
            //new_date.month = t.date.month+t.sched.frequency; new_date.year = t.date.year;
            //while (new_date.month>12) { 
            //    new_date.month -= 12; 
            //    new_date.year++;
            //}

			//if the day was initially the last in its month, then make next occurance to have the day as last in its month
			if(t.date.day == (UInt16) DaysInMonth(t.date.month,t.date.year)) 
				new_date.day=DaysInMonth(new_date.month, new_date.year);				
			else //if the new month has less days then the initial month, adjust 
				new_date.day=(t.date.day>(UInt16) DaysInMonth(new_date.month,new_date.year)) ? DaysInMonth(new_date.month,new_date.year) : t.date.day;
			break;
		case rep_year:
			new_date.month=t.date.month;
			new_date.year=t.date.year+t.sched.frequency;
			if(t.date.day == (UInt16) DaysInMonth(t.date.month,t.date.year)) 
				new_date.day=DaysInMonth(new_date.month, new_date.year);				
			else //if the new month has less days then the initial month, adjust 
				new_date.day=(t.date.day>(UInt16) DaysInMonth(new_date.month,new_date.year)) ? DaysInMonth(new_date.month,new_date.year) : t.date.day;
			break;
	}
   //record the date change
	t.date.day=new_date.day;
	t.date.month=new_date.month;
	t.date.year=new_date.year;
   
	//now save the date changes
	packRecord(&t,rec_h);
	    
	if( t.sched.end_by_date) {
        if(compareDates(t.sched.end_date,t.date) <=0 ) {
		    deleteRecord(id,true); //if the new date is past the specified end date, delete scheduled transaction; we need to maintain the balances
		    return true;
        }
        return false;
    }
	
    if(t.sched.repeat_times==-1) return false; //repeat forever; record is never deleted

	if( !t.sched.end_by_date && !t.sched.paused) { //if we stop after a given number of transactions and recording not paused
		if(t.sched.repeat_times>0) { // decrease the no of times we have left to repeat
			t.sched.repeat_times--;
			packRecord(&t,rec_h);
		} 
		if(t.sched.repeat_times==0) { //only makes sense to check this after a decrement
			deleteRecord(id,true); //if we have repeated the asked no of times, delete record
			return true;
		}
	}
	return false;
}

Int32      TransactionsDB::getScheduledTotal(UInt32 account_uid)
{
   UInt32                     total = 0;
   UInt16                      record_no;

  	for(record_no=0;DmQueryNextInCategory(ref,&record_no,getScheduledCategory());record_no++) {
         total+=getAmount(getRecordUid(record_no), account_uid); //getAmount will simply return 0 if account not affected
   }
   return total;
}

Boolean TransactionsDB::schedule(Boolean force)
{   //this is very inneficient, but at least it works :-)
    DateType today;
    DateSecondsToDate(TimGetSeconds(),&today);

    if(!force) //if we are not forced to continue
        if(compareDates(preferences.get()->last_update,today)>=0) //and we don't need to update anything yet
            return false;

    Boolean rez=false;
    while(scheduleNextRecord()) { rez = true; }
        preferences.get()->last_update=today;
    if(rez) { //if we made changes we should resort the transactions db.This is a temporary (and rather unelegant) fix
        DmInsertionSort(getRef(), (DmComparF *) compareRecords,0);
    }
    return rez;
}


UInt32 TransactionsDB::recordScheduledRecord(UInt32 record_uid, bool mod_date)
{
	ErrFatalDisplayIf (!isScheduled (getRecordId (record_uid)),"Record given is not scheduled");
	transactionStructure temp;
	
	getUnpackedRecord (record_uid,&temp); //get the next scheduled record
	UInt16 count = temp.num_splits;
	temp.is_scheduled = false;
	if(mod_date) {
		DateSecondsToDate(TimGetSeconds(),&temp.date);
	}

	//first create a new transaction record
	UInt32 new_uid = getRecordUid(newRecord(&temp,temp.sched.to_be_cleared));
	for (UInt16 index=splits_db->locateFirstSplit(record_uid);count>0;count--,index++) {
		splitStructure              split;
					
		//cycle through the splits and duplicate each one of them
		splits_db->getUnpackedRecord(index,&split);
		split.trans_uid = new_uid; //make split point to our new transaction
		splits_db->newRecord(&split,temp.sched.to_be_cleared);
	}	
	return new_uid;
}

Boolean TransactionsDB::scheduleNextRecord()
{
	DateType today;
	DateSecondsToDate(TimGetSeconds(),&today);
	
	/*	now we go through all the records and if we find one which has an associated transaction which is due
	today or on a previous date, we record that transaction and get its next date */
	
	Boolean 			  updated=false;

  	for(UInt16 record_no=0;DmQueryNextInCategory(ref,&record_no,getScheduledCategory());record_no++) {        
       	transactionStructure	temp;

		getUnpackedRecord(record_no,&temp); //get the next scheduled record
		//operations on the record can change its id which in turn can lead to calculateNextDate being
		//executed on the wrong record. this is why we need to use the uid to go past the loop below
		UInt32 record_uid = getRecordUid(record_no);
        
	      //for each record repeat updating until there are no updates left to do
		if (compareDates(temp.date,today) <=0 ) {	//if record date is in the past or today
			if(! temp.sched.paused) { //if paused, do not record new transactions
				recordScheduledRecord (record_uid,false);
			}
			updated=true;
            //now calculate the next date when the transaction is due
            calculateNextDate(getRecordId(record_uid));
		}
	}
	return updated;
}

Int32 TransactionsDB::getTransactionByPayeeAndAcc(Char* p, UInt16 start_id)
{
	if (StrLen(p) < 1) return -1; //don't do anything with empty strings
	
	transactionStructure	ts;	
	UInt32 rec_uid;
	for (UInt16 record_id=start_id;record_id<DmNumRecords(ref);record_id++) {
		getUnpackedRecord(record_id,&ts);
		rec_uid = getRecordUid(record_id);
		if(! isScheduled(record_id)) { //only work on the string if the transaction is not scheduled
			if(!StrNCaselessCompare(p, ts.payee,StrLen(p)) && getAmount(rec_uid, accounts_db->getSelectedUid())!=0) {
				return rec_uid;
			}
		}
	}
	return -1;
}

Int32 TransactionsDB::getTransactionByPayee(Char* p, UInt16 start_id, Boolean want_cur_acc)
{
	if (StrLen(p) < 1) return -1; //don't do anything with empty strings
	
	transactionStructure	ts;	
	for (UInt16 record_id=start_id;record_id<DmNumRecords(ref);record_id++) {
		getUnpackedRecord(record_id,&ts);

		if(! isScheduled(record_id)) { //only work on the string if the transaction is not scheduled
			if(!StrNCaselessCompare(p, ts.payee,StrLen(p))) {
				if (want_cur_acc) { //if the transaction has to be in the same acc
					//try and find it
					Int32 next_uid = getTransactionByPayeeAndAcc(p,record_id);
					//if found return it; otherwise return our first match
					if (next_uid >=0) return next_uid;
				}
				return getRecordUid(record_id);
			}
		}
	}
	return -1;
}
