/**	$Revision: 1.28 $
	Last updated: $Date: 2002/12/03 22:44:33 $

	Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "TransactionsForm.h"
#include "EditAccountForm.h"
#include "CoinsPref.h"
#include "GenericInputForm.h"
#include "FormManager.h"
#include "FreeCoinsRsc.h"

TransactionsForm* TransactionsForm::_instance = NULL;
TransactionsForm trans_form;

TransactionsForm* TransactionsForm::instance()
{
	if(_instance == NULL) {
		_instance = &trans_form;
	}
	return _instance;
}
TransactionsForm::TransactionsForm() : Form (TRANSACTIONS_FORM)
{
	_edit_trans_form = EditTransactionForm::instance();
}

void	TransactionsForm::initCat()
{
	ControlPtr	ctl;	
	// Set the label of the category trigger.
	ctl = (ControlPtr) GetObjectPtr (TRANS_CAT_TRIGGER);
	CategoryGetName (transactions_db->getRef(), transactions_db->current_cat, _cat_name);
	CategorySetTriggerLabel(ctl, _cat_name);
}

void	TransactionsForm::selectCat()
{
 	UInt16		category=transactions_db->current_cat;
	Boolean   categoryEdited;

	//have to do this due to some odd behaviour 
	frm = FrmGetActiveForm();
	
	categoryEdited=CategorySelect(transactions_db->getRef(),frm, TRANS_CAT_TRIGGER,TRANS_CAT_LIST,false,&category,
								  _cat_name,4,(osVerGe(palmos_3_5)) ? categoryHideEditCategory:0);
	if(categoryEdited || category!=transactions_db->current_cat) {
		transactions_db->current_cat=category;
		if(preferences.get()->remember_trans_view) {
			preferences.get()->trans_view=category;
		}
		//start from the beginning each time the category is changed
		_trans_tbl.top_visible_item=0;
		_trans_tbl.hideScrollArrows();
	}
}

void	TransactionsForm::adjustClearedAccBalance()
{
	accountStructure	ac;
	MemHandle		h;
	Char			r[9];
	UInt16			rez;
	Int32			new_bal;

	MemHandle str_handle = DmGetResource (strRsc,USER_INPUT_STR);
	MemHandle str1_handle = DmGetResource (strRsc,NEW_CLEARED_BAL_STR);
	GenericInputForm g(reinterpret_cast<char*> (MemHandleLock (str_handle)),reinterpret_cast<char*> (MemHandleLock (str1_handle)),r,9);
	rez=g.doDialog(frm);
	if (rez) {
		new_bal = String2Pence(r);
		h = accounts_db->getUnpackedRecord(&ac);
		new_bal *= (ac.credit_account ? -1 : 1);	
		ac.balance = new_bal;
		accounts_db->packRecord(&ac,h);
	}
	MemHandleUnlock (str_handle);
	DmReleaseResource (str_handle);
	MemHandleUnlock (str1_handle);
	DmReleaseResource (str1_handle);
}

void	TransactionsForm::adjustAccBalance()
{
	accountStructure ac;
	Char r[9];
	UInt16 rez;
	Int32 new_bal;

	MemHandle str_handle = DmGetResource (strRsc,USER_INPUT_STR);
	MemHandle str1_handle = DmGetResource (strRsc,NEW_BAL_STR);
	GenericInputForm g(reinterpret_cast<char*> (MemHandleLock (str_handle)),reinterpret_cast<char*> (MemHandleLock (str1_handle)),r,9);
    rez=g.doDialog(frm);
	if (rez) {
		new_bal = String2Pence(r);
		accounts_db->getUnpackedRecord(&ac);
		new_bal *= (ac.credit_account ? -1 : 1);	
		accounts_db->updateBalance(accounts_db->getSelectedUid(),new_bal);			
	}    
	MemHandleUnlock (str_handle);
	DmReleaseResource (str_handle);
	MemHandleUnlock (str1_handle);
	DmReleaseResource (str1_handle);
}



void	TransactionsForm::displayTotals()
{
	Char* s;
	RectangleType rp;
	Int32 factor;

	accountStructure ac;
      	
	accounts_db->getUnpackedRecord(&ac);
    factor = (ac.credit_account || ac.type == INCOME)?-1:1;

	//display cleared balance
	if(transactions_db->current_cat==transactions_db->getScheduledCategory()) {
		// have to display total of scheduled trans	
		factor *= transactions_db->getScheduledTotal(accounts_db->getSelectedUid());
	} else {
		factor *= ac.cleared_balance;
	}
   
	s=Pence2String(factor);
	
	RctSetRectangle(&rp,115,129,45,11);
	WinEraseRectangle(&rp,0);
	DrawCharsRightAlign(s, &rp);
	MemPtrFree(s);

	//display balance
	factor = ac.balance;
	if(transactions_db->current_cat==transactions_db->getScheduledCategory()) {
		// have to display expected total of scheduled trans	... do nothing
	} else {
		if (ac.credit_account || ac.type == INCOME) factor = -factor;
	}
	s=Pence2String(factor);

	RctSetRectangle(&rp,70,129,45,11);
	WinEraseRectangle(&rp,0);
	DrawCharsRightAlign(s, &rp);
	MemPtrFree(s);	

	//add some lines at the bottom
	WinDrawLine(0,129,160,129);
	WinDrawLine(0,140,160,140);	
	
}
Boolean TransactionsForm::handleOpenEvent()
{
	accountStructure as;

	//set the form title to the name of the account
	accounts_db->getUnpackedRecord(&as);
	FrmSetTitle(frm,as.name);
      
	if(preferences.get()->disp_payee) 
		FrmSetControlGroupSelection(frm,1,TRANS_OPT_PAYEE);
	else FrmSetControlGroupSelection(frm,1,TRANS_OPT_ACCOUNT);

	transactions_db->current_cat = preferences.get()->trans_view;				
	_trans_tbl.initTable();
	transactions_db->unselectRecord();

	//display the category name
	initCat();
      
	//draw form
	if(!osVerGe(palmos_3_5)) FrmEraseForm(frm);
	FrmDrawForm(frm);
	displayTotals();
	_trans_tbl.showScrollArrows();	
	return true;
}

Boolean TransactionsForm::handleKeyDownEvent(WChar key_id, UInt16 modifiers)
{
	ControlPtr         ptr;
	switch(key_id) {
		case pageUpChr:			// Scroll up key presed
			//link the hard button functionality to the soft up/down controls
			ptr = (ControlPtr) GetObjectPtr(TRANS_PGUP);
			CtlHitControl(ptr);
			break;								
		case pageDownChr:		// Scroll down key presed						
			ptr = (ControlPtr) GetObjectPtr(TRANS_PGDN);
			CtlHitControl(ptr);
			break;
		default:
			return false;
	};
	return true;
}

Boolean TransactionsForm::handleCtlSelectEvent (UInt16 control_id)
{
	switch(control_id) {
		case TRANS_NEW:
			handleNewTransaction();
			break;
		case TRANS_ACCTS:
			//cancel the current selection
			transactions_db->unselectRecord();
			_trans_tbl.hideScrollArrows();
			accounts_db->unselectRecord();
			FrmUpdateForm(caller_form_id,0);				
			FrmReturnToForm(0);
			break;				
		case TRANS_CAT_TRIGGER:		//Select a category
			selectCat();
			FrmUpdateForm(form_id,0);
			break;
		case TRANS_OPT_PAYEE: //switch to payee vieew
			if(preferences.get()->disp_payee) break;
			else preferences.get()->disp_payee=true;
			FrmUpdateForm(form_id,1);
			break;
		case TRANS_OPT_ACCOUNT: //switch to account view
			if(!preferences.get()->disp_payee) break;
			else preferences.get()->disp_payee=false;
			FrmUpdateForm(form_id,1);
			break;
		case TRANS_PGUP:
			if(_trans_tbl.pageUp()) FrmUpdateForm(form_id,1);
			break;								
		case TRANS_PGDN:		// Scroll down control presed?	
			if(_trans_tbl.pageDown()) FrmUpdateForm(TRANSACTIONS_FORM,1);
			break;
		default:
			return false;
	}
	return true;
}

Boolean TransactionsForm::handleTblSelectEvent(tblSelect &tbl_select)
{
	transactions_db->selectRecord(TblGetRowID((TablePtr) GetObjectPtr(TRANS_TABLE),tbl_select.row));
	if(tbl_select.column == TransactionsTable::cleared_col) {
		Boolean clr = TblGetItemInt((TablePtr) GetObjectPtr(TRANS_TABLE),tbl_select.row,tbl_select.column);
		if(clr && !transactions_db->isCleared(transactions_db->getSelectedRecord())) {
			transactions_db->clearRecord(transactions_db->getSelectedRecord());
		} else 
			if(!clr && transactions_db->isCleared(transactions_db->getSelectedRecord())) {
				transactions_db->unclearRecord(transactions_db->getSelectedRecord());
			} else  //exit witouth updating the database
				return true;
		
		//we don't need to update the form in the cl/uncl category
		MemHandle str_handle = DmGetResource (strRsc,CL_UNCL_STR);
		if(transactions_db->current_cat!=transactions_db->findCategory(reinterpret_cast<char*> (MemHandleLock (str_handle))) && transactions_db->current_cat!=transactions_db->getScheduledCategory())		
			FrmUpdateForm(form_id,0);		
		else displayTotals();
		transactions_db->unselectRecord();
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
	} else {
		handleTransactionDetails();
	}
	return true;
}


Boolean TransactionsForm::handleMenuEvent(UInt16 menu_item_id)
{
	switch (menu_item_id) {
		case TRANS_MENU_NEW:
			handleNewTransaction();
			break;					
		case TRANS_MENU_ADJST_CLR_BAL:
			adjustClearedAccBalance();
			FrmUpdateForm(form_id,0);
			//cancel the current selection
			transactions_db->unselectRecord();
			break;		
		case TRANS_MENU_ADJST_BAL:
			adjustAccBalance();
			FrmUpdateForm(form_id,0);
			//cancel the current selection
			transactions_db->unselectRecord();
			break;		
		case TRANS_MENU_ACC_DETAILS:
			if (preferences.get()->def_currency == 0) {
				MemHandle str_handle = DmGetResource (strRsc,NO_DEF_CUR_STR);
				FrmCustomAlert(GENERIC_INFO, reinterpret_cast<char*> (MemHandleLock (str_handle)), NULL, NULL);
				MemHandleUnlock (str_handle);
				DmReleaseResource (str_handle);
				FrmUpdateForm (form_id,0);
				break;
			}
			EditAccountForm::instance()->setRecordIsNew(false);
			if(!osVerGe(palmos_3_5)) FrmEraseForm(frm);
			popup(EditAccountForm::instance());
			//cancel the current selection
			transactions_db->unselectRecord();
			if(! accounts_db->isRecordSelected()) {
				FrmUpdateForm(caller_form_id,0);
				FrmReturnToForm(caller_form_id);
			}
			break;			
		case TRANS_MENU_ABOUT: {
			FormPtr f = FrmInitForm(ABOUT_FORM);
			FrmSetActiveForm(f);
			FrmDoDialog(f);
			if (frm) {
				FrmSetActiveForm(frm);
				FrmDeleteForm(f);
			}
			FrmUpdateForm (form_id,0);	
			break;
		}
		case TRANS_MENU_HELP:
			FrmHelp(TRANS_HELP);
			FrmUpdateForm (form_id,0);
			break;
				
		default:
			return false;
	}
	return true;
}

Boolean TransactionsForm::handleCloseEvent()
{
	_trans_tbl.hideScrollArrows();
	return Form::handleCloseEvent();
}

Boolean TransactionsForm::handleUpdateEvent(UInt16 update_code)
{	
	_trans_tbl.loadData();

	if(update_code == 0 || update_code == frmRedrawUpdateCode) {
		accountStructure as;
		accounts_db->getUnpackedRecord(&as);
		FrmSetTitle(frm,as.name);
		
		//display the category name
		initCat();
		
		//draw form
		if (!osVerGe(palmos_3_5)) FrmEraseForm(frm);
		FrmDrawForm(frm);
		//display the running totals
		displayTotals();
	}
	if(update_code & 1) {
		TblMarkTableInvalid((TablePtr)GetObjectPtr(TRANS_TABLE));
		TblRedrawTable((TablePtr)GetObjectPtr(TRANS_TABLE));
	}			

	if(update_code==2) {
		//display the running totals
		displayTotals();
		return true;
	}

	_trans_tbl.showScrollArrows();
	
	return true;
}

void TransactionsForm::handleTransactionDetails()
{
	_edit_trans_form->setRecordIsNew(false);
	// Please do not touch this line, it fixes bug #435607
	_trans_tbl.unhighlight();
	popup (_edit_trans_form);
}                  

void TransactionsForm::handleNewTransaction()
{
	transactions_db->unselectRecord();
 	_edit_trans_form->setRecordIsNew(true);
	popup (_edit_trans_form);
}            
