/**	$Revision: 1.16 $
	Last updated: $Date: 2002/09/25 21:18:01 $

	Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef __TRANSACTIONS_FORM__
#define __TRANSACTIONS_FORM__

#include <PalmOS.h>
#include "Form.h"
#include "TransactionsTable.h"
#include "EditTransactionForm.h"


/** functions and vars that manipulate the form which deals with transactions
	\note the duplicate menu and button events are handled by handle* functions do eliminate code duplication
*/
class TransactionsForm : public Form
{
public:
	///constructor; dows nothing
	TransactionsForm() FORM_SECTION2;
 	/**	creates a new transaction updating the balance in the new account */
	void	adjustAccBalance() FORM_SECTION2;	
	/**	updates the expected balance for the account */
	void	adjustClearedAccBalance() FORM_SECTION2;
	/**	updates the totals for the current account (if necessary) and draws them on the screen */
	void	displayTotals() FORM_SECTION2;
	///returns singleton reference to this object
	static TransactionsForm* instance() FORM_SECTION2;

	/**	called when a form update event occurs (public in order to be able to force updates)
	    @param update_code the update code signalled */
	Boolean handleUpdateEvent(UInt16 update_code) FORM_SECTION2;
 private:
	///reference to this object
	static TransactionsForm* _instance;
	//pointer to the active edit  transaction form
	EditTransactionForm* _edit_trans_form;  
	///used to store the current category name 
	Char _cat_name [dmCategoryLength];
	
	///the transactions table
	TransactionsTable _trans_tbl;

	///called when key down event occurs
	Boolean handleKeyDownEvent(WChar key_id, UInt16 modifiers) FORM_SECTION2;
	///called when form open event occurs
	Boolean handleOpenEvent() FORM_SECTION2;
	/**	called when ctl select event occurs
            @param control_id id of the control for which the event was generated */
	Boolean handleCtlSelectEvent(UInt16 control_id) FORM_SECTION2;
	/**  called when a table select event occurs 
		 @param tbl_select forwarded tblSelect event data   */
	Boolean handleTblSelectEvent(tblSelect& tbl_select) FORM_SECTION2;
	/**	called when a menu event occurs */
	Boolean handleMenuEvent(UInt16 menu_item_id) FORM_SECTION2;

	///this function is called when the form is closed
	Boolean	handleCloseEvent() FORM_SECTION2;

	/// init the category trigger
	void	initCat() FORM_SECTION2;

	///handle category selection
	void	selectCat() FORM_SECTION2;
   
	/**	executes the necessary operations to start creating a new transaction */
	void handleNewTransaction() FORM_SECTION2;
         
	/**	handles transaction details	*/
	void handleTransactionDetails() FORM_SECTION2;
};

#endif
