/**	$Revision: 1.12 $
	Last updated: $Date: 2002/09/25 21:18:01 $
	
	Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#define __TRANSACTIONS_TABLE_C__

#include "TransactionsTable.h"

#include "TransactionsDB.h"
#include "AccountsDB.h"
#include "MiscFunc.h"
#include "CoinsPref.h"

#include "FreeCoinsRsc.h"

void TransactionsTable::drawCurrency(void* table, Int16 row, Int16 column, RectangleType* bounds)
{
	Char					*s;
	accountStructure 		ac;
	UInt32 trans_uid, acc_uid;
	Int32 amount;

	accounts_db->getUnpackedRecord(&ac);
	trans_uid = transactions_db->getRecordUid(TblGetRowID((TablePtr)table,row));
	acc_uid = accounts_db->getSelectedUid();

	amount = transactions_db->getAmount(trans_uid, acc_uid);
	if (ac.credit_account || ac.type==INCOME) amount *= -1;
	
	s=Pence2String(amount);

	DrawCharsRightAlign(s, bounds);
	//free memory allocated by Pence2String
	MemPtrFree(s);	

   // mark the transaction if it is unbalanced
   if(!transactions_db->isBalanced(trans_uid)) {
		MemHandle res_h = DmGetResource('Tbmp', BMP_UNBALANCED);
		BitmapType *bitmap = (BitmapType *) MemHandleLock(res_h);
		WinDrawBitmap(bitmap, bounds->topLeft.x, bounds->topLeft.y+2);
		MemHandleUnlock(res_h);
		DmReleaseResource(res_h);
   }
}

void TransactionsTable::drawDate(void* table, Int16 row, Int16 column, RectangleType* bounds)
{
	Char		s[6];
	DateType d;
	UInt8 num1, num2;

	transactions_db->getDate(TblGetRowID((TablePtr) table,row), &d);
	if (MiscFunc::bDayFirst) {
		num1 = d.day; num2 = d.month;
	} else {
		num2 = d.day; num1 = d.month;
	}
	StrPrintF(s, "%02d%c%02d", num1, MiscFunc::cDateSep, num2);

	WinDrawChars(s,5,bounds->topLeft.x,bounds->topLeft.y);
}


void TransactionsTable::drawAcc(void* table, Int16 row, Int16 column, RectangleType* bounds)
{
	transactionStructure 	ts;
	UInt32 					trans_uid, peer_uid;
	Char 					buf[50];
	char*					str = NULL;
	bool					need_unlock=false;
	MemHandle 				str_handle = NULL;

	transactions_db->getUnpackedRecord(TblGetRowID((TablePtr) table,row),&ts);
	
	if(preferences.get()->disp_payee) // display the payee
		str = ts.payee;
	else { //display the peer account
		switch (ts.num_splits) {
			case 1: 
				str_handle = DmGetResource (strRsc,NOT_SET_STR);
				need_unlock=true;
				break;
			case 2:
				trans_uid = transactions_db->getRecordUid(TblGetRowID((TablePtr) table,row));
				peer_uid = transactions_db->getPeerAcc(
					trans_uid, accounts_db->getSelectedUid());
				accounts_db->getFullNameTruncLeftToFit(peer_uid,
					TblGetColumnWidth((TablePtr) table, dest_col), buf);
				str = buf;
				break;
			default: 
				str_handle = DmGetResource (strRsc,D_SPLIT_D_STR);
				need_unlock = true;
				break;
		}
	}	
	if(need_unlock) {
		str=reinterpret_cast<char*> (MemHandleLock (str_handle));
		WinDrawChars(str,StrLen(str),bounds->topLeft.x,bounds->topLeft.y);
		MemHandleUnlock (str_handle);
		DmReleaseResource (str_handle);
	} else {
		WinDrawChars(str,StrLen(str),bounds->topLeft.x,bounds->topLeft.y);
	}
}

Boolean TransactionsTable::recordIsValid(UInt16 id)
{
	UInt32 trans_uid;
	
	//check the category
	if((transactions_db->current_cat==transactions_db->getClUnclCategory()) && ( \
		transactions_db->getRecordCategory(id)!=transactions_db->getClearedCategory() && \
		transactions_db->getRecordCategory(id)!=transactions_db->getUnclearedCategory())) return false; 
	//check accounts
	trans_uid = transactions_db->getRecordUid(id);
	if(transactions_db->getAmount(trans_uid, accounts_db->getSelectedUid())==0)
		return false;
	return true;
}

void TransactionsTable::loadData()
{
	UInt16 	recordNo,row,i;
	UInt16	categ;

	row = 0;
	i = 0;

	initColumns();
	
	categ=transactions_db->current_cat;
	if (categ==transactions_db->getClUnclCategory())
		categ = dmAllCategories;

	for (recordNo=0;DmQueryNextInCategory(transactions_db->getRef(),&recordNo,categ); recordNo++) {
		if (!recordIsValid(recordNo)) continue;
		if (i++ < top_visible_item) continue;
		if (row>=rows) continue;
		TblSetRowID(_table, row, recordNo);
		TblSetItemInt(_table, row, cleared_col,
			transactions_db->isCleared(recordNo));
		TblSetRowUsable(_table, row, true);
		row++;
	}

	setRowsUnusable(row);
	total_items = i;
}

TransactionsTable::TransactionsTable() : Table(TRANS_TABLE, TRANS_PGUP, TRANS_PGDN)
{ }

void TransactionsTable::initColumns()
{
	//the numbers are drawn with our custom procedure
	TblSetCustomDrawProcedure(_table, date_col, drawDate);
	TblSetCustomDrawProcedure(_table, dest_col, drawAcc);
	TblSetCustomDrawProcedure(_table, amount_col, drawCurrency);

	//set style for all cells
	for (UInt16 row = 0; row < rows; row++) {
		TblSetItemStyle(_table, row, date_col, customTableItem);
		TblSetItemStyle(_table, row, dest_col, customTableItem);
		TblSetItemStyle(_table, row, amount_col, customTableItem);
	}
	
 	//set columns as usable
	TblSetColumnUsable(_table, date_col, true);
	
	//display the cleared column if needed
	if(preferences.get()->disp_cleared_col) {
		TblSetColumnUsable(_table, cleared_col,true);
	} else {
		//dont display the column
		TblSetColumnUsable(_table, cleared_col,false);
		TblSetColumnWidth(_table,dest_col,87);
	}
	//display the amount and dest column
 	TblSetColumnUsable(_table, dest_col, true);	
	TblSetColumnUsable(_table, amount_col, true);	
}
